import java.io.*;
import java.net.*;
import java.util.*;


class  NetTestCommand  extends  Thread
{
    private static final int  SOCKET_TIMEOUT_TIME = 7200000; // ms.

    private static final String  ARGV_START     =
	":Command-Argv-Start:";
    private static final String  ARGV_END       =
	":Command-Argv-End:";
    private static final String  STDOUT_START   =
	":Command-STDOUT-Start:";
    private static final String  STDOUT_END     =
	":Command-STDOUT-End:";
    private static final String  STDERR_START   =
	":Command-STDERR-Start:";
    private static final String  STDERR_END     =
	":Command-STDERR-End:";
    private static final String  EXIT_VAL_START =
	":Command-ExitVal-Start:";
    private static final String  EXIT_VAL_END   =
	":Command-ExitVal-End:";


    private Socket	    client_socket;
    private BufferedReader  client_socket_in;
    private PrintWriter	    client_socket_out;
    private ComErrorStore   test_com_error_store;
    private String	    peer_hostname;
    private SysProp	    sys_prop;
    private Param	    param;
    private LogOut	    log_out;
    private int		    id;


    NetTestCommand (Socket   client_socket,
		    SysProp  sys_prop,
		    Param    param,
		    LogOut   log_out,
		    int      id)
    {
	super ();

	this.client_socket	  = client_socket;
	this.client_socket_in	  = null;
	this.client_socket_out	  = null;
	this.test_com_error_store = null;
	this.peer_hostname	  = null;
	this.sys_prop		  = sys_prop;
	this.param		  = param;
	this.log_out		  = log_out;
	this.id			  = id;

	setDaemon (true);
    }


    public void  run ()
    {
	try
	{
	    client_socket.setSoTimeout (SOCKET_TIMEOUT_TIME);
	    setSocketStreams	       ();
	    checkPeerName	       ();

	    String []  argv = readArgv ();

	    checkArgv (argv);
	    runWrite  (argv);
	}
	catch (Exception  except)
	{
	    log_out.printLine (except.getMessage (), id);
	}
	finally
	{
	    try
	    {
		closeSocketStreams ();
		closeClientSocket  ();
	    }
	    catch (IOException  io_except)
	    {
		log_out.printLine (io_except.getMessage (), id);
	    }

	    stopTestComErrorStore ();
	}
    }


    private void  setSocketStreams ()  throws  IOException
    {
	InputStream   socket_input  = client_socket.getInputStream  ();
	OutputStream  socket_output = client_socket.getOutputStream ();

	client_socket_in = new BufferedReader (
				   new InputStreamReader (
					   socket_input));

	client_socket_out = new PrintWriter (
				    new OutputStreamWriter (
					    socket_output),
				    true);
    }


    private void  closeSocketStreams ()  throws  IOException
    {
	if (client_socket_in != null)
	{
	    client_socket_in.close ();
	    client_socket_in = null;
	}

	if (client_socket_out != null)
	{
	    client_socket_out.close ();
	    client_socket_out = null;
	}
    }


    private void closeClientSocket ()  throws  IOException
    {
	if (client_socket != null)
	{
	    client_socket.close ();
	    client_socket = null;

	    if (peer_hostname == null)
		log_out.printLine ("Connection closed", id);
	    else
		log_out.printLine ("Connection " + peer_hostname +
				   " closed",
				   id);
	}
    }


    private void  checkPeerName ()  throws  UnknownHostException,
					    HostRefusedException
    {
	int port = client_socket.getPort ();

	InetAddress peer_inet_address = client_socket.getInetAddress ();

	peer_hostname = peer_inet_address.getHostName ();

	String  peer_ip_addr_str = peer_inet_address.getHostAddress ();

	String  peer_connect_str;
	String  peer_host_str;

	if (peer_hostname == null)
	{
	    peer_hostname    = peer_ip_addr_str;
	    peer_connect_str = peer_ip_addr_str;
	    peer_host_str    = peer_ip_addr_str;
	}
	else
	{
	    peer_connect_str = peer_hostname +
			       "(" + peer_ip_addr_str + ")";
	    peer_host_str    = peer_hostname;
	}

	log_out.printLine ("Connecting " + peer_connect_str +
			   " on port " + port,
			   id);

	if ( !param.acceptHost (peer_hostname	)  &&
	     !param.acceptHost (peer_ip_addr_str) )
	{
	    throw new HostRefusedException (
			  "Connection from peer " + peer_host_str +
			  " refused");
	}
    }


    private String []  readArgv ()  throws  IOException
    {
	boolean  debug = param.getDebug ();

	Vector	 arg_vector = new Vector ();
	boolean  read_argv  = false;
	boolean	 do_read    = true;
	String	 line;

	while ( do_read  &&
		(line = client_socket_in.readLine ()) != null )
	{
	    line.trim ();

	    if (debug)
	    {
		log_out.printLine ("<< " + line, id);
	    }

	    if (line.equals (ARGV_START))
	    {
		read_argv = true;
	    }
	    else if (line.equals (ARGV_END))
	    {
		do_read = false;
	    }
	    else if (read_argv  &&  line.length () > 0)
	    {
		arg_vector.addElement (line);
	    }
	}

	String []  argv = new String [arg_vector.size ()];

	arg_vector.copyInto (argv);

	return argv;
    }


    private void  checkArgv ( String []  argv
			    )
			    throws  InvalidTestCommandException
    {
	if ( argv.length < 1  ||  !param.allowBaseCommand (argv[0]) )
	{
	    throw new InvalidTestCommandException (
			  "Refusing invalid net test command: " +
			  getStringFromArray (argv));
	}

	int  argv_length = argv.length;
	int  index	 = 0;

	while (index < argv_length)
	{
	    if ( !checkArgChars (argv [index++].toCharArray ()) )
	    {
		throw new InvalidTestCommandException (
			      "Refusing net test command for invalid " +
			      "arguments: " +
			      getStringFromArray (argv));
	    }
	}
    }


    private static boolean  checkArgChars (char []  arg_chars)
    {
	int  arg_chars_length = arg_chars.length;
	int  index	      = 0;

	while (index < arg_chars_length)
	{
	    char  arg_c = arg_chars [index++];

	    if ( !( arg_c >= 'A'  &&  arg_c <= 'Z'
		    ||
		    arg_c >= 'a'  &&  arg_c <= 'z'
		    ||
		    arg_c >= '0'  &&  arg_c <= '9'
		    ||
		    arg_c == '_'
		    ||
		    arg_c == '-'
		    ||
		    arg_c == '/'
		    ||
		    arg_c == ','
		    ||
		    arg_c == '.' ) )
	    {
		return false;
	    }
	}

	return true;
    }


    private void  runWrite ( String []  argv
			   )
			   throws  IOException, InterruptedException
    {
	boolean debug = param.getDebug ();

	argv [0] = param.getProgramDir	     () +
		   sys_prop.getFileSeparator () + argv [0];

	if (sys_prop.isWindowsOS ())
	    argv = getWindowsArgv (argv);

	log_out.printLine ("Running: " + getStringFromArray (argv),
			   id);

	Runtime  runtime = Runtime.getRuntime ();

	Process  test_com_proc = runtime.exec (argv);

	InputStream  test_com_input = test_com_proc.getInputStream ();
	InputStream  test_com_error = test_com_proc.getErrorStream ();

	test_com_error_store = new ComErrorStore (test_com_error,
						  log_out,
						  id);

	test_com_error_store.start ();

	BufferedReader  test_com_reader;

	test_com_reader = new BufferedReader (
				  new InputStreamReader (
					  test_com_input));

	if (debug)
	{
	    log_out.printLine (">>", id);
	    log_out.printLine (">> " + STDOUT_START, id);
	}

	client_socket_out.println (STDOUT_START);

	String  line;

	while ( (line = test_com_reader.readLine ()) != null )
	{
	    if (debug)
		log_out.printLine (">> " + line, id);

	    client_socket_out.println (line);
	}

	test_com_reader.close ();
	test_com_proc.waitFor ();

	int  exit_val = test_com_proc.exitValue ();

	if (debug)
	    log_out.printLine (">> " + STDOUT_END, id);

	client_socket_out.println (STDOUT_END);

	String  test_com_error_content =
		    test_com_error_store.getContent ();

	if (test_com_error_content.length () > 0)
	{
	    int  nr_lines = test_com_error_store.getNrLines ();

	    printTestComErrorContent (test_com_error_content,
				      nr_lines,
				      argv);
	}

	if (debug)
	{
	    log_out.printLine (">> " + EXIT_VAL_START, id);
	    log_out.printLine (">> " + exit_val,       id);
	    log_out.printLine (">> " + EXIT_VAL_END,   id);
	}

	client_socket_out.println (EXIT_VAL_START + "\n" +
				   exit_val	  + "\n" +
				   EXIT_VAL_END);
    }


    private static String []  getWindowsArgv (String []  argv)
    {
	int  argv_length = argv.length;
	int  win_offset	 = 3;

	String []  win_argv = new String [argv_length + win_offset];

	win_argv [0] = "perl";
	win_argv [1] = "-w";
	win_argv [2] = "-S";
	win_argv [3] = argv [0] + ".pl";

	for (int index = 1;  index < argv_length;  index++)
	    win_argv [index + win_offset] = argv [index];

	return win_argv;
    }


    private void  printTestComErrorContent (String     content,
					    int        nr_lines,
					    String []  argv)
    {
	log_out.printLine ("Command contains \"stderr\" content: " +
			   getStringFromArray (argv),
			   id);

	client_socket_out.println (STDERR_START + "\n" +
				   content	+
				   STDERR_END);

	log_out.printLines (content, nr_lines, id);
    }


    private static String  getStringFromArray (String []  array)
    {
	StringBuffer  array_sb = new StringBuffer ();

	int  array_length = array.length;
	int  index	  = 0;

	while (index < array_length)
	{
	    if (index > 0)
		array_sb.append (" ");

	    array_sb.append (array [index++]);
	}

	return array_sb.toString ();
    }


    private void  stopTestComErrorStore ()
    {
	if (test_com_error_store != null)
	{
	    test_com_error_store.stop ();
	    test_com_error_store = null;
	}
    }
}
