/* ------------------------------------------------------------------ */
/*		  C o n t r o l B u t t o n . j a v a		      */
/*		  ===================================		      */
/*								      */
/*   This file contains the source of the "ControlButton" class. It   */
/*   is used for a control button in the DataPlotWindow" class.	      */
/* ------------------------------------------------------------------ */


import java.awt.*;


/**
 * This class is used for a control button in the
 * <CODE>DataPlotWindow</CODE> class. The parameters used by this class
 * are defined in the <CODE>ControlParameters</CODE> class. An instance
 * of the <CODE>ControlStatusHandler</CODE> should be specified to
 * display status messages when this component is entered / exited.
 *
 *    @see  DataPlotWindow
 *    @see  ControlParameters
 *    @see  ControlStatusHandler
 *
 *    @author  Hans Blom
 *
************************************/
class  ControlButton  extends  Button
/***********************************/
{
    /*
     * Declare the class variables.
     */

    private ControlStatusHandler  status_handler;   // Handle status m.
    private String		  mouse_enter_mess; // Mouse enter mess.


    /**
     * Contructor.
     *
     *    @param  status_handler     The status handler which handles
     *				     status messages.
     *
     *    @param  label.	     The label of this control button.
     *
     *    @param  mouse_enter_mess   The status messages shown when the
     *				     mouse enters this button.
     *
    *****************************************************/
    ControlButton (ControlStatusHandler  status_handler,
		   String		 label,
		   String		 mouse_enter_mess)
    /****************************************************/
    {
	/*
	 * Call the "super()" constructor to set the label of this
	 * button.
	 */

	super (label);

	/*
	 * Set the class instances.
	 */

	this.status_handler   = status_handler;
	this.mouse_enter_mess = mouse_enter_mess;

	/*
	 * Set the control parameters for this button.
	 */

	ControlParameters.setComponent (this);

    } // ControlButton ()


    /**
     * Set the status message which is shown when a mouse enters this
     * button.
     *
     *    @param  mouse_enter_mess   The status message which is shown
     *				     when a mouse enters this button.
     *
    ****************************************************/
    void  setMouseEnterMessage (String  mouse_enter_mess)
    /***************************************************/
    {
	this.mouse_enter_mess = mouse_enter_mess;
    }


    /**
     * Get the status message which is shown when a mouse enters this
     * button.
     *
     *    @return   The status message which is shown when a mouse
     *		    enters this button.
     *
    ******************************/
    String  getMouseEnterMessage ()
    /*****************************/
    {
	return mouse_enter_mess;
    }


    /**
     * Handle the event when a mouse enters this button. Show the
     * corresponing status message when the mouse did enter it. In all
     * other situatons the super method is called to handle the event.
     *
     *    @param  evt   The event that caused the action.
     *
     *    @param  x	The <I>X</I>&nbsp;position of the mouse pointer.
     *
     *    @param  y	The <I>Y</I>&nbsp;position of the mouse pointer.
     *
     *    @return   <CODE>true</CODE> when the event was handled;
     *		    <CODE>false</CODE> otherwise.
     *
    **************************************/
    public boolean  mouseEnter (Event  evt,
				int    x,
				int    y)
    /*************************************/
    {
	if (evt.target == this)    // Target is the plot canvas.
	{
	    /*
	     * Let the "ControlStatusHandler" handler interface show the
	     * status message.
	     */

	    status_handler.showMessage (mouse_enter_mess);

	    return true;
	}
	else    // Another target: handled by the super method.
	{
	    return super.mouseEnter (evt, x, y);
	}

    } // mouseEnter ()


    /**
     * Handle event when a mouse exited this button. Clear the status
     * message when the mouse did enter it. In all other situatons the
     * super method is called to handle the event.
     *
     *    @param  evt   The event that caused the action.
     *
     *    @param  x	The <I>X</I>&nbsp;position of the mouse pointer.
     *
     *    @param  y	The <I>Y</I>&nbsp;position of the mouse pointer.
     *
     *    @return   <CODE>true</CODE> when the event was handled;
     *		    <CODE>false</CODE> otherwise.
     *
    ************************************/
    public boolean  mouseExit (Event evt,
    			       int   x,
			       int   y)
    /***********************************/
    {
	if (evt.target == this)    // Target is the plot canvas.
	{
	    /*
	     * Let the "ControlStatusHandler" handler interface clear
	     * the status message.
	     */
	     
	    status_handler.showMessage ("");

	    return true;
	}
	else    // Another target: handled by the super method.
	{
	    return super.mouseExit (evt, x, y);
	}

    } // mouseExit ()

} // class ControlButton
