/* ------------------------------------------------------------------ */
/*		  C o n t r o l C h o i c e . j a v a		      */
/*		  ===================================		      */
/*								      */
/*   This file ontains the source of the "ControlChoice" class. It    */
/*   is used for a control choice in the DataPlotWindow" class.	      */
/* ------------------------------------------------------------------ */


import java.awt.*;


/**
 * This class is used for a control choice in the
 * <CODE>DataPlotWindow</CODE> class. The parameters used by this class
 * are defined in the <CODE>ControlParameters</CODE> class. An instance
 * of the <CODE>ControlStatusHandler</CODE> should be specified to
 * display status messages when this component is entered / exited. From
 * this interface an empty implementation is specified by this class.
 *
 *    @see  DataPlotWindow
 *    @see  ControlParameters
 *    @see  ControlStatusHandler
 *
 *    @author  Hans Blom
 *
**********************************************************************/
class  ControlChoice  extends  Choice  implements  ControlStatusHandler
/*********************************************************************/
{
    /*
     * Declare the class variables.
     */

    private ControlStatusHandler  status_handler;   // Handle status m.
    private String		  mouse_enter_mess; // Mouse enter mess.


    /**
     * Constructor.
     *
     *    @param  items		     The selector items.
     *
     *	  @param  default_item	     The default item which is selected 
     *				     in the choice.
     *
     *    @param  mouse_enter_mess   The message wich is displayed when
     *				     the mouse enters this component.
     *
    ******************************************/
    ControlChoice (String []  items,
		   String     default_item,
		   String     mouse_enter_mess)
    /*****************************************/
    {
	/*
	 * Default use the empty status handler.
	 */

	status_handler  = this;  // Default use this empty status hndl.

	/*
	 * Set the status message when the mouse enters this control
	 * choice.
	 */

	this.mouse_enter_mess = mouse_enter_mess;

	/*
	 * Set the control parameters for this choice.
	 */

	ControlParameters.setComponent (this);

	/*
	 * Add the items to the choice selector.
	 */

	int  nr_items = items.length;

	for (int index = 0;  index < nr_items;  index++)
	    addItem (items [index]);

	/*
	 * Select the default item in the choice selector.
	 */

	select (default_item);

    } // ControlChoice ()


    /**
     * Set the control status handler to display status messages from
     * this component.
     *
     *    @param  status_handler   The specified status handler.
     *
    ************************************************************/
    void  setStatusHandler (ControlStatusHandler  status_handler)
    /***********************************************************/
    {
	this.status_handler = status_handler;
    }


    /**
     * Get the control status handler to display status messages from
     * this component.
     *
     *    @return   The used status handler.
     *
    ****************************************/
    ControlStatusHandler  getStatusHandler ()
    /***************************************/
    {
	return status_handler;
    }


    /**
     * Empty implementation of the <CODE>showMessage()</CODE> from the
     * <CODE>ControlStatusHandler</CODE> interface. This intreface is
     * used to display the status messages from this control component.
     *
     *    @param  message   The control component status message to
     *			    show.
     *
     /****************************************/
     public void  showMessage (String  message)
     /****************************************/
     {
     }


     /**
     * Handle the event when a mouse enters this choice. Show the
     * corresponing status message when the mouse did enter it. In all
     * other situatons the super method is called to handle the event.
     *
     *    @param  evt   The event that caused the action.
     *
     *    @param  x	The <I>X</I>&nbsp;position of the mouse pointer.
     *
     *    @param  y	The <I>Y</I>&nbsp;position of the mouse pointer.
     *
     *    @return   <CODE>true</CODE> when the event was handled;
     *		    <CODE>false</CODE> otherwise.
     *
    **************************************/
    public boolean  mouseEnter (Event  evt,
				int    x,
				int    y)
    /*************************************/
    {
	if (evt.target == this)    // Target is the plot canvas.
	{
	    /*
	     * Let the "ControlStatusHandler" handler interface show the
	     * status message.
	     */

	    status_handler.showMessage (mouse_enter_mess);

	    return true;
	}
	else    // Another target: handled by the super method.
	{
	    return super.mouseEnter (evt, x, y);
	}

    } // mouseEnter ()


    /**
     * Handle event when a mouse exited this choice. Clear the status
     * message when the mouse did enter it. In all other situatons the
     * super method is called to handle the event.
     *
     *    @param  evt   The event that caused the action.
     *
     *    @param  x	The <I>X</I>&nbsp;position of the mouse pointer.
     *
     *    @param  y	The <I>Y</I>&nbsp;position of the mouse pointer.
     *
     *    @return   <CODE>true</CODE> when the event was handled;
     *		    <CODE>false</CODE> otherwise.
     *
    ************************************/
    public boolean  mouseExit (Event evt,
    			       int   x,
			       int   y)
    /***********************************/
    {
	if (evt.target == this)    // Target is the plot canvas.
	{
	    /*
	     * Let the "ControlStatusHandler" handler interface clear
	     * the status message.
	     */
	     
	    status_handler.showMessage ("");

	    return true;
	}
	else    // Another target: handled by the super method.
	{
	    return super.mouseExit (evt, x, y);
	}

    } // mouseExit ()

} // Class ControlChoice
