/* ------------------------------------------------------------------ */
/*	      C o n t r o l P a r a m e t e r s . j a v a	      */
/*	      ===========================================	      */
/*								      */
/*   This file contains the source of the final "ControlParameters"   */
/*   class. It is used to define and set some parameters which are    */
/*   used in the control components from the "DataPlotWindow" class.  */
/* ------------------------------------------------------------------ */


import java.awt.*;


/**
 * This statis class is used to define and set some parameters which are
 * used in the control components from the <CODE>DataPlotWindow</CODE>
 * class.
 *
 *    @see  DataPlotWindow
 *
 *    @author  Hans Blom
 *
*****************************/
final class  ControlParameters
/****************************/
{
    /**
     * Define the font used by the control components.
     */

    static final Font  FONT = new Font ("Serif", Font.PLAIN, 14);

    /**
     * Define the background color used by the control parameters.
     */

    static final Color  BACKGROUND = Color.gray;

    /**
     * Define the foreground color used by the control parameters.
     */

    static final Color  FOREGROUND = Color.white;

    /*
     * Define the name of the OS.
     */

    private static final String  OS_NAME = System.getProperty (
					      "os.name");

    /*
     * Define the OS name prefix of Macintosh.
     */

    private static final String  MAC_PRE_OS_NAME = "Mac";


    /**
     * Empty constructor. It is made private, because there is no need
     * to create new instances of this static class.
     *
    ****************************/
    private  ControlParameters ()
    /***************************/
    {
    }


    /**
     * Set the parameters of the specified control component. When the
     * background and foreground colors of the control component could
     * not be set to the required values, the original colours are
     * restores. Don not set background and foreground colors at
     * Macintosh, because they have a.o. differently formed buttons.
     *
     *    @param  control  The control component from which the
     *			   parameters should be set.
     *
    *********************************************/
    static void  setComponent (Component  control)
    /********************************************/
    {
	control.setFont (FONT);

	/*
	 * Do not set colours at Macintosh OS.
	 */

	if ( OS_NAME.startsWith (MAC_PRE_OS_NAME) )
	{
	    return;
	}

	/*
	 * Inquire the original foreground and background colours.
	 */

	Color  orig_background = control.getBackground ();
	Color  orig_foreground = control.getForeground ();

	/*
	 * Set the control background and foreground colours.
	 */

	control.setBackground (BACKGROUND);
	control.setForeground (FOREGROUND);

	/*
	 * Restore the original colours when the background and/or
	 * foreground colour could not be set.
	 */

	if ( !( control.getBackground ().equals (BACKGROUND)  &&
		control.getForeground ().equals (FOREGROUND) ) )
	{
	    control.setBackground (orig_background);
	    control.setForeground (orig_foreground);
	}

    } // setComponent ()

} // class ControlParameters
