/* ------------------------------------------------------------------ */
/*		 D a t a P l o t W i n d o w . j a v a		      */
/*		 =====================================		      */
/*								      */
/*   This file contains the source of the "DataPlotWindow" class.     */
/*   It is used to define the plot window.			      */
/* ------------------------------------------------------------------ */


import java.util.*;
import java.awt.*;
import user.plot.*;


/**
 * This class is used to define the plot window. When a plot window is
 * displayed is is contained in the corresponding instance of the
 * <CODE>DataPlotFrame</CODE> class, however all window events are
 * delegated to this class. The plot window contains:
 *
 * <UL>
 *
 * <LI>A panel with control buttons. It may also contain externally
 *     defined control components.</LI>
 *
 * <LI>A legend canvas. A scrollbar is added when the legend is too
 *     large.</LI>
 *
 * <LI>The plot canvas.</LI>
 *
 * <LI>The plot traces are shown in the <CODE>PlotCanvas</CODE>.
 *     Therefore it is required to implement the
 *     <CODE>PlotLegendHandler</CODE> interface.</LI>
 *
 * <LI>A label showing the status of the plot. Therefore this class
 *     implements the <CODE>PlotStatusHandler</CODE> interface.</LI>
 *
 * </UL>
 * </P>
 *
 * <P>
 * The <CODE>PlotTraceHandler</CODE> interface is implemented to connect
 * the legend index to a plot trace.
 * </P>
 *
 * <P>
 * For the time rounding at the axis ticks the <CODE>DataHandler</CODE>
 * interface is implemented.
 * </P>
 *
 * <P>
 * In general there is extra buffer storage for the next plot such that
 * attributes can be added without affecting the current plot.
 *
 *    @see  DataPlotFrame
 *
 *    @author  Hans Blom
 *
*******************************************************/
class  DataPlotWindow  implements  PlotLegendHandler,
				   PlotTraceHandler,
				   PlotStatusHandler,
				   ControlStatusHandler,
				   DataHandler
/******************************************************/
{
    /**
     * Define the plot trace colors.
     */

    static final Color []  TRACE_COLORS =
    {
	Color.red,
	Color.green,
	Color.blue,
	Color.yellow,
	Color.pink,
	Color.magenta,
	Color.orange,
	Color.cyan,
	new Color (165,  42,  42),    // Brown.
	new Color (184, 134,  11),    // Dark Golden Rod.
	new Color (104,  34, 139)     // Dark Orchid 4.
    };

    /**
     * Define the default boundary for a white foreground: when the
     * green intensity is below this value this is the case; else a
     * black foreground is used.
     */

    static final int  DFT_GREEN_WHITE_FGND_BND = 160;

    /*
     * Define the colors connected to the boundary described above.
     */

    private static final Color  LOW_INT_FOREGROUND  = Color.white;
    private static final Color  HIGH_INT_FOREGROUND = Color.black;

    /**
     * Define the default legend background which is used when the
     * corresponding data are not plotted.
     */

    static final Color  DFT_NO_PLOT_LEGEND_BGND = Color.lightGray;

    /**
     * Define the default legend foreground which is used when the
     * corresponding data are not plotted.
     */

    static final Color  DFT_NO_PLOT_LEGEND_FGND = Color.gray;

    /*
     * Define the initial width and height of the plot canvas.
     */

    private static final int  INIT_PLOT_WIDTH  = 550;
    private static final int  INIT_PLOT_HEIGHT = 450;

    /*
     * Define the min width and height of the plot canvas.
     */

    private static final int  MIN_PLOT_WIDTH  = 55;
    private static final int  MIN_PLOT_HEIGHT = 45;

    /*
     * Define several borders used in the plot window frame.
     */

    private static final int  PLOT_BORDER  = 5; // For diff. components.
    private static final int  AREA_BORDER  = 5; // Around plot canvas.
    private static final int  INTER_BORDER = 4; // For conn. components.

    /*
     * Define several plot window frame colors.
     */

    private static final Color  BACKGROUND	     = Color.white;
    private static final Color  PLOT_BACKGROUND	     = Color.lightGray;
    private static final Color  PLOT_AREA_BACKGROUND = Color.white;

    /*
     * Define the default font. Use the same font used in the control
     * components.
     */

    private static final Font  FONT = ControlParameters.FONT;

    /*
     * Define the control button labels.
     */

    private static final String  CLOSE_BUTTON_LABEL	 = "Close";
    private static final String  MARKERS_BUTTON_LABEL	 = "Markers";
    private static final String  NO_MARKERS_BUTTON_LABEL = "No Markers";
    private static final String  JOIN_BUTTON_LABEL	 =
				     "Join Points";
    private static final String  NO_JOIN_BUTTON_LABEL	 = "No Join";

    /*
     * Define the control button status messages.
     */

    private static final String  CLOSE_BUTTON_STATUS	  =
	"Close the plot window.";
    private static final String  MARKERS_BUTTON_STATUS	  =
	"Show markers.";
    private static final String  NO_MARKERS_BUTTON_STATUS =
	"Do not show markers.";
    private static final String  JOIN_BUTTON_STATUS	  =
	"Join points by lines.";
    private static final String  NO_JOIN_BUTTON_STATUS	  =
	"Do not join points.";

    /*
     * Define the initial value of the plot status label.
     */

    private static final String  INIT_STATUS_LABEL = "Plot Status";

    /*
     * Define the font used by the status label.
     */

    private static final Font  STATUS_FONT = new Font ("Serif",
						       Font.PLAIN,
						       12);

    /*
     * Define the # lines used in the date ticks and the corresponding
     * indices for the single time and date lines.
     */

    private static final int  NR_DATE_TICK_LINES = 2;
    private static final int  TIME_TICK_INDEX	 = 0;
    private static final int  DATE_TICK_INDEX    = 1;

    /*
     * Define some scaling factors used in the date and time rounding.
     */

    private static final double  MINUTES = (double) DateFormat.MINUTES;
    private static final double  HOURS	 = (double) DateFormat.HOURS;
    private static final double  DAYS	 = (double) DateFormat.DAYS;

    /*
     * Define the rounding boundary: values are rounded or ignored when
     * they are above this value. The reason is that they are expected
     * to be below this value, because we deal here with the double
     * representation of integer values.
     */

    private static final double  ROUND_BOUND = 1.0E-4;


    /*
     * Define the class variables.
     */

    private DataPlotFrame	plot_frame;	    // The displ. frame.
    private GridBagLayout       grid_bag;	    // To layout comp.
    private PlotCanvas          plot;		    // Plot canvas.
    private String		plot_title;	    // Title plot frame.
    private String		x_axis_label;	    // Label X axis.
    private String		y_axis_label;	    // Label Y axis.
    private LegendCanvas        legend;		    // Legend canvas.
    private boolean		show_no_plot_legends;// Show no plot l.?
    private LegendScrollbar	legend_scrollbar;   // Legend scrollbar.
    private boolean		use_legend_scrollbar;// Use legend sbar?
    private Font	        font;		    // Default font.
    private Font	        status_font;	    // Font status lab.
    private Panel		control_panel;	    // Control panel.
    private GridBagLayout	control_grid_bag;   // Ctrl pan. layout.
    private ControlButton	close_button;	    // Close frame but.
    private ControlButton	markers_button;	    // Show markers b.?
    private ControlButton	join_button;	    // Join lines but.?
    private ExternPlotControls  extern_controls;    // For ext. ctrl's.
    private ExternPlotControls  buffer_extern_controls;// Buffer ext. c.
    private Label	        status_label;	    // Plot status lab.
    private int			green_white_fgnd_bnd;// Gr. wh. fg. bnd.
    private Color		no_plot_legend_bgnd;// No plot leg. bgd.
    private Color		no_plot_legend_fgnd;// No plot leg. fgd.
    private Vector	        pot_plot_data;	    // Pot. data plot.
    private Vector	        buffer_pot_plot_data;// Data buff. plot.
    private Vector	        plot_labels;	    // Labels in legend.
    private Vector	        buffer_plot_labels; // Labels buff. leg.
    private Vector		host_keys;	    // Host keys;
    private Vector		buffer_host_keys;   // Buffer host keys.
    private Hashtable		no_plot_host_keys;  // Excl. host keys.
    private Hashtable		buffer_no_plot_host_keys;// Buff exc. k.
    private int	   []		legend_to_data_index; // Leg. -> data i.
    private int	   []		legend_to_trace_index;// Leg. -> tr.  i.
    private int	   []		trace_to_legend_index;// Tr.  -> leg. i.
    private String []	        dates;		    // Dates in plot.
    private String []	        times;		    // Times in plot.
    private String []	        buffer_dates;	    // Dates buffer.
    private String []	        buffer_times;	    // Times buffer.
    private String []	        date_marker;	    // Date ticks mark.
    private DateFormat		marker_date_format; // Date at tck. mrk.
    private long		time_offset;	    // Tick time offset.
    private long		buffer_time_offset; // Buffer time offs.
    private Hashtable		date_id_table;	    // Date ID table.
    private Vector		date_vector;	    // D.V. used by IDs.
    private int			date_vector_index;  // Curr. index D.V.
    private Hashtable		buffer_date_id_table;// Buff. date ID t.
    private Vector		buffer_date_vector; // Buffer Date Vect.
    private boolean		round_date_grid;    // Date grid ticks?
    private boolean		use_markers;	    // Point markers?
    private boolean		use_join;	    // Join points?
    private int			new_plot_width;	    // Width new plot.
    private int			new_plot_height;    // Height new plot.


    /**
     * Constructor.
     *
     *    @param  plot   The plot canvas to be placed in the plot frame.
     *
    ********************************/
    DataPlotWindow (PlotCanvas  plot)
    /*******************************/
    {
	this.plot = plot;     // Set the plot canvas.

	initPlotParameters ();    // Initialise the plot parameters.
	initComponents	   ();    // Init. the comp.for the plot frame.
	setData		   ();    // Allocate and initialise the data.
	setInterfaces	   ();    // Set the plot canvas interfaces.
	
    } // DataPlotWindow ()


    /***
     * Initialise the plot parameters.
     *
    **********************************/
    private void  initPlotParameters ()
    /*********************************/
    {
	/*
	 * Resize the plot canvas to the initial sizes such that it
	 * won't have initially zero sizes.
	 */

	plot.resize (INIT_PLOT_WIDTH, INIT_PLOT_HEIGHT);

	/*
	 * Initalise the plot frame related variables to their undefined
	 * values.
	 */

	clearPlotFrameVar ();

	/*
	 * Initialise some plot presentation related parameters.
	 */
	
	round_date_grid = true;     // Round ticks at the date grid.
	use_markers     = false;    // Do not mark plot points.
	use_join        = true;	    // Join points with line pieces.

	/*
	 * Set below some plot parameters to their default values.
	 */

	/*
	 * Set the default boundary where the white foreground color is
	 * used.
	 */

	green_white_fgnd_bnd = DFT_GREEN_WHITE_FGND_BND;

	/*
	 * Set the default background and foreground of the legends from
	 * which the corresponding plot traces will not be add to the
	 * plot canvas.
	 */

	no_plot_legend_bgnd = DFT_NO_PLOT_LEGEND_BGND;
	no_plot_legend_fgnd = DFT_NO_PLOT_LEGEND_FGND;

	/*
	 * Default show legends not plotted.
	 */

	show_no_plot_legends = true;

	/*
	 * Set empty plot title and axis labels.
	 */
	 
	plot_title   = "";
	x_axis_label = "";
	y_axis_label = "";

	/*
	 * Initialise the new plot sizes to zero: i.e. they are still
	 * undefined.
	 */

	new_plot_width  = 0;
	new_plot_height = 0;

    } // initPlotParameters ()


    /**
     * Initialise the plot components.
     *
    ******************************/
    private void  initComponents ()
    /*****************************/
    {
	/*
	 * Initialise the following settings of the plot canvas:
	 *
	 *   1. The background of the whole plot canvas.
	 *
	 *   2. The background of its plot area.
	 *
	 *   3. The font of the plot canvas.
	 */

	plot.setBackground     (PLOT_BACKGROUND);	  // 1.
	plot.setAreaBackground (PLOT_AREA_BACKGROUND);    // 2.
	plot.setFont	       (FONT);			  // 3.

	/*
	 * Create and initialise the settings of the legend components.
	 */

	legend = new LegendCanvas (this);    // Create legend canvas.

	legend.setFont (FONT);    // Set legend font.

	legend_scrollbar     = new LegendScrollbar (); // Set leg. sbar.
	use_legend_scrollbar = false;		       // Not yet used.

	/*
	 * Create and intitalise the settings of the default control
	 * panel buttons.
	 */

	close_button   = new ControlButton (this,
					    CLOSE_BUTTON_LABEL,
					    CLOSE_BUTTON_STATUS);
	markers_button = new ControlButton (this,
					    getMarkersButtonLabel  (),
					    getMarkersButtonStatus ());
	join_button    = new ControlButton (this,
					    getJoinButtonLabel  (),
					    getJoinButtonStatus ());

	/*
	 * Initially no external controls are available at the plot and
	 * at the storage buffer for the next plot.
	 */

	extern_controls	       = null;
	buffer_extern_controls = null;

	/*
	 * Set the status label and its properties.
	 */

	status_label = new Label (INIT_STATUS_LABEL); // Set init. text.

	status_label.setFont	  (STATUS_FONT);     // Set font.
	status_label.setAlignment (Label.CENTER);    // Center text.

	/*
	 * Set the initial width and height of the status label In the
	 * height the inter-line spacing will be skipped.
	 */

	FontMetrics  font_metrics = status_label.getFontMetrics (
				        STATUS_FONT);

	int  label_width = font_metrics.stringWidth (INIT_STATUS_LABEL);
	int  font_height = font_metrics.getMaxAscent  () +
			   font_metrics.getMaxDescent ();

	status_label.resize (label_width, font_height);

    } // initComponents ()


    /**
     * Set the new sizes which are used to resize the data plot canvas
     * before it is (re)showed in <CODE>showPlot()</CODE>. Default to
     * the initial size when a specified size is smaller equal the
     * minimum size
     *
     *    @param  new_plot_width    The new plot width which is set when
     *				    the plot window is (re)showed.
     *
     *    @param  new_plot_height   The new plot height which is set
     *    			    when the plot window is (re)showed.
     *
     *    @see  DataPlotWindow#showPlot
     *
    ******************************************/
    void  setNewPlotSize (int  new_plot_width,
			  int  new_plot_height)
    /*****************************************/
    {
	if (new_plot_width > MIN_PLOT_WIDTH)
	    this.new_plot_width = new_plot_width;
	else
	    this.new_plot_width = MIN_PLOT_WIDTH;

	if (new_plot_height > MIN_PLOT_HEIGHT)
	    this.new_plot_height = new_plot_height;
	else
	    this.new_plot_height = MIN_PLOT_HEIGHT;
    }


    /**
     * Allocate and initialise the data used in the plot canvas.
     * Initialy the various data are empty.
     *
    ***********************/
    private void  setData ()
    /**********************/
    {
	plot_labels		 = null;
	buffer_plot_labels	 = new Vector ();
	pot_plot_data		 = null;
	buffer_pot_plot_data	 = new Vector ();
	host_keys		 = null;
	buffer_host_keys	 = new Vector ();
	no_plot_host_keys	 = null;
	buffer_no_plot_host_keys = null;
	legend_to_data_index	 = null;
	legend_to_trace_index	 = null;
	trace_to_legend_index	 = null;
	dates			 = null;
	times			 = null;
	buffer_dates		 = null;
	buffer_times		 = null;
	time_offset		 = -1;
	buffer_time_offset	 = -1;
	date_id_table		 = null;
	date_vector		 = null;
	date_vector_index	 = 0;
	buffer_date_id_table	 = null;
	buffer_date_vector	 = null;
	date_marker		 = new String     [NR_DATE_TICK_LINES];
	marker_date_format	 = new DateFormat ();
    
    } // setData ()


    /**
     * Set the interfaces, required by the plot frame and used by the
     * plot canvas. They are all implemented by this class instance.
     *
    *****************************/
    private void  setInterfaces ()
    /****************************/
    {
	plot.setPlotTraceHandler  (this);
	plot.setPlotStatusHandler (this);
	plot.setXDataHandler	  (this);
    }


    /**
     * Set if at the date plot ticks rounding should be done at the plot
     * date grid or that the date values should be interpolated at the
     * plot ticks.
     *
     *    @param  round_date_grid   True: use date grids; false:
     *				    interpolate dates.
     *
    ************************************************/
    void  setRoundDateGrid (boolean  round_date_grid)
    /***********************************************/
    {
	this.round_date_grid = round_date_grid;
    }


    /**
     * Return <CODE>true</CODE> when date plot ticks rounding is done at
     * the plot date grid or <CODE>false</CODE> when the date values are
     * interpolated at the plot ticks.
     *
     *    @return   The type of date tick rounding.
     *
    ***************************/
    boolean  getRoundDateGrid ()
    /**************************/
    {
	return round_date_grid;
    }


    /**
     * Return the background color of the specified legend. When shown
     * it is the color of the corresponding plot trace. Note that the
     * trace color is also connected to the legend index. Otherwise it
     * is the background color of legend of the plot trace not shown.
     * This method should be implemented for the
     * <CODE>PlotLegendHandler</CODE> interface.
     *
     *    @param  legend_index   The index of the legend to specify.
     *
     *    @return   The legend background color.
     *
    ****************************************************/
    public Color  getLegendBackground (int  legend_index)
    /***************************************************/
    {
	int  trace_index = legend_to_trace_index [legend_index];

	if (trace_index >= 0)    // Plot trace shown.
	{
	    return plot.getTraceColor (
			    legend_index % plot.getNrTraceColors ());
	}
	else    // Plot trace not shown.
	{
	    return no_plot_legend_bgnd;
	}
    }


    /**
     * Return the foreground color of the specified legend. When shown
     * it depends from the green value of the color of the corresponding
     * plot trace. This is also the color of the corresponding legend
     * background. When not shown it is the foreground of the legend of
     * the plot trace not shown. This method should be implemented for
     * the <CODE>PlotLegendHandler</CODE> interface.
     *
     *    @param  legend_index   The index of the legend to specify.
     *
     *    @return   The legend background color.
     *
    ****************************************************/
    public Color  getLegendForeground (int  legend_index)
    /***************************************************/
    {
	int  trace_index = legend_to_trace_index [legend_index];
	
	if (trace_index >= 0)    // Plot trace shown.
	{
	    Color  legend_background =
		       plot.getTraceColor (legend_index %
					   plot.getNrTraceColors ());

	    if (legend_background.getGreen () < green_white_fgnd_bnd)
		return LOW_INT_FOREGROUND;
	    else
		return HIGH_INT_FOREGROUND;
	}
	else    // Plot trace not shown.
	{
	    return no_plot_legend_fgnd;
	}
    }


    /**
     * Return if markers should be used in the legends. This method
     * should be implemented for the <CODE>PlotLegendHandler</CODE>
     * interface.
     *
     *    @return   True when markers should be used; false otherwise.
     *
    *************************************/
    public boolean  getLegendUseMarkers ()
    /************************************/
    {
	return plot.getUseMarkers ();
    }


    /**
     * Get the marker used in the specified legend. When shown it is the
     * marker used in the corresponding plot trace. Note that the trace
     * marker is always connected to the legend index. Otherwise
     * <CODE>PlotMarker.NONE_MARKER</CODE> is returned, so no marker
     * will be drawn. This method should be implemented for the
     * <CODE>PlotLegendHandler</CODE> interface.
     *
     *    @param  legend_index   The index of the legend to specify.
     *
     *    @return   The marker of the specify legend.
     *
    *********************************************************/
    public PlotMarker  getLegendPlotMarker (int  legend_index)
    /********************************************************/
    {
	int  trace_index = legend_to_trace_index [legend_index];
	
	if (trace_index >= 0)    // Plot trace shown.
	{
	    return plot.getMarker (legend_index % plot.getNrMarkers ());
	}
	else    // Plot trace not shown.
	{
	    return PlotMarker.NONE_MARKER;
	}
    }


    /**
     * Use the legend index as the index of the plot trace. This
     * function is the implementation of the
     * <CODE>PlotTraceHandler</CODE> interface.
     *
     *    @param  trace_data_index   The specified data index from the
     *				     pot trace.
     *
     *    @return   The legend index, to be used for the plot trace
     *		    index.
     *
    ************************************************/
    public int  getTraceIndex (int  trace_data_index)
    /***********************************************/
    {
	return trace_to_legend_index [trace_data_index];
    }


    /**
     * Show a message in the plot status label when the mouse enters the
     * legend canvas. This method should be implemented for the
     * <CODE>PlotLegendHandler</CODE> interface.
     *
    *******************************/
    public void  legendMouseEnter ()
    /******************************/
    {
	changeStatusLabel (
	    "Left: (no) trace; "	   +
	    "Alt+Cl. / Middle: plot all; " +
	    "Meta+Cl. / Right: all / plot legends.");
    }


    /**
     * Clear the message in the plot status label when the mouse exits
     * the legend canvas. This method should be implemented for the
     * <CODE>PlotLegendHandler</CODE> interface.
     *
    ******************************/
    public void  legendMouseExit ()
    /*****************************/
    {
	changeStatusLabel ("");
    }
	

    /**
     * Handle the release of the mouse in the specified legend box. The
     * following actions are possible.
     *
     * <DL>
     *
     * <DT><B>Left mouse click</B></DT>
     * <DD>If the plot trace, specified by this legend, was shown in the
     *	   plot window, it wil will <EM>not</EM> be shown when the plot
     *	   is redisplayed, and reverse.</DD>
     *
     * <DT><B>Alt key&nbsp;+ click or middle mouse click</B></DT>
     * <DD>Show all plot traces for this data type.
     *
     * <DT><B>Meta key&nbsp;+ click or right mouse click</B></DT>
     * <DD>Toggle the display of the non-plotted legends, if any. When
     *	   all legends are displayed at the moment, always display the
     *	   non-plotted legends.</DD>
     *
     * </DT>
     *
     * Reshow the plot after the action has been handled. Return if the
     * action has been handled. This method should be implemented for
     * the <CODE>PlotLegendHandler</CODE> interface.
     *
     *    @param  event		 The mouse up event.
     *
     *    @param  legend_index   The index of the legend box where the
     *				 mouse was released. The index is set
     *				 to&nbsp;-1 when the mouse was not
     *				 released in a legend box.
     *
     *    @return   True whe the action has been handled; false
     *		    otherwise.
     *
    **************************************************/
    public boolean  legendMouseUp (Event  event,
				   int	  legend_index)
    /*************************************************/
    {
	if ( (event.modifiers & Event.ALT_MASK) != 0 )  // Middle mouse.
	{
	    /*
	     * Show all plot traces by clearing the hashtable with host
	     * keys not to plot. Also reset the show no plot legend
	     * flag.
	     */

	    no_plot_host_keys.clear ();
	    show_no_plot_legends = true;

	    /*
	     * Do not redisplay the plot, by returning "true" here, when
	     * all plot traces are already shown.
	     */
	    
	    if (plot.getDataSize () == pot_plot_data.size ())
		return true;
	}
	else if ( (event.modifiers & Event.META_MASK) != 0 ) // Right m.
	{
	    /*
	     * (Don't) Show the not plotted legends by:
	     *
	     *    1. Set the flag "true" when all plot traces are shown.
	     *
	     *    2. Toggle the flag.
	     */

	    if (plot.getDataSize () == pot_plot_data.size ())   // 1.
	    {
		show_no_plot_legends = true;

		/*
		 * Nothing to be done anymore, because the plot window
		 * stays unchanged. Return "true" that the event has
		 * been used.
		 */

		return true;
	    }
	    else    // 2.
	    {
		show_no_plot_legends = !show_no_plot_legends;
	    }
	}
	else    // Left mouse release.
	{
	    /*
	     * Do nothing when the mouse was not relased within a legend
	     * box. Return "false" to denote that the event was not
	     * used.
	     */

	    if (legend_index == -1)
		return false;
	    
	    /*
	     * Convert the legend index to a data index.
	     */

	    int  data_index = legend_to_data_index [legend_index];

	    /*
	     * Obtain the host key of the corresponding plot trace.
	     */
	 
	    String host_key = (String) host_keys.elementAt (data_index);

	    /*
	     * Add the host key to the plot traces not show when it was
	     * shown, or remove it when it was shown at the moment.
	     */

	    if (no_plot_host_keys.containsKey (host_key))  // Show the
	    {						   // trace the
		no_plot_host_keys.remove (host_key);	   // next time.
	    }
	    else    // Don't show the plot the next time.
	    {
		/*
		 * Do nothing when there is no more than one plot trace
		 * left: at least do not remove the last plot trace.
		 * Event used: return "true".
		 */

		 if (plot.getDataSize () <= 1)
		    return true;

		/*
		 * Add the host key to the hastable containing the host
		 * keys of the traces tp exclude. The host key is put as
		 * key AND as value.
		 */

		no_plot_host_keys.put (host_key, host_key);
		//		       Key	 Value

	    } // Don't show the plot the next time.

	} // Left mouse click.

	/*
	 * Reset the plot data traces to show and the legends to reflect
	 * the changes in the plot.
	 */

	setPlotDataAndLegendsShown ();

	/*
	 * Redisplay the plot with the new settings.
	 */

	showPlot ();

	/*
	 * Return "true" to reflect that the action event was used.
	 */

	return true;

    } // legendMouseUp ()


    /**
     * Change the cursor to the crosshair cursor when the plot canvas is
     * entered and show a plot canvas message in the plot status label.
     * This function is the implementation of the
     * <CODE>PlotStatusHandler</CODE> interface.
     *
     *    @param  message	 The plot canvas message to display.
     *
     *    @param  canvas_state   The enter, within or exit state of the
     *				 plot canvas.
     *
    **********************************************/
    public void  showMessage (String  message,
			      int     canvas_state)
    /*********************************************/
    {
	/*
	 * Change the cursor when the plot canvas is entered / exited.
	 * To be sure check synchronized if the plot frame exists.
	 */

	switch (canvas_state)
	{
	    case PlotCanvas.ENTER :    // Enter: show crosshair cursor
				       // to emphasize zooming.

		synchronized (this)    // For plot frame shown status.
		{
		    if (plot_frame != null)    // Plot frame is shown.
		    {
			plot_frame.setCursor (Frame.CROSSHAIR_CURSOR);
		    }
		}
		break;

	    case PlotCanvas.EXIT :    // Exit: change to default cursor.

		synchronized (this)
		{
		    if (plot_frame != null)
		    {
			plot_frame.setCursor (Frame.DEFAULT_CURSOR);
		    }
		}
		break;

	    default :    // Do nothing in other situations.

		break;
	}

	/*
	 * Change the status label with the specified message.
	 */

	changeStatusLabel (message);
    
    } // showMessage ()


    /**
     * Show a control component status message in the plot status label
     * when a control component is entered or exited. This function is
     * the implementation of the <CODE>ControlStatusHandler</CODE>
     * interface.
     *
     *    @param  message   The control component message to display.
     *
    *****************************************/
    public void  showMessage (String  message)
    /****************************************/
    {
	/*
	 * Change the status label with the specified message.
	 */

	changeStatusLabel (message);
    }


    /**
     * Change the status label with the specified message.
     *
     *    @param  message   The specified message.
     *
    ************************************************/
    private void  changeStatusLabel (String  message)
    /***********************************************/
    {
	/*
	 * Change only the status label when the plot frame currently is
	 * shown. Change the label synchronized with this class instance
	 * such that the plot frame will not be modified during this
	 * operation.
	 */

	synchronized (this)    // For plot frame (not) shown status.
	{
	    if (plot_frame != null)    // Plot frame is being shown.
	    {
		/*
	 	 * Change the label text. This requires first removing
		 * it from the window frame. After changing the label
		 * text the status label is again added to the frame and
		 * the plot frame is again packed and shown to let it
		 * redisplay the changed label.
		 */

		plot_frame.remove    (status_label);
		status_label.setText (message);
		addStatusLabel	     ();
		// plot_frame.pack	     ();
		plot_frame.show	     ();

	    } // Plot frame is being shown.

	} // Synchronized this class instance for plot frame status.
    
    } // changeStatusLabel ()


    /**
     * Round the specified value with a rounding step not larger than
     * the specified range and return the rounded value. This function
     * is part of the implementation of the <CODE>DataHandler</CODE>
     * interface for the dates axis. Therefore it depends from the type
     * of date scaling.
     *
     *    @param  value   The value to round.
     *
     *    @param  range   The rounding step may never exceeds this
     *			  range.
     *
     *    @return   The rounded value.
     *
    ***********************************/
    public double  round (double  value,
			  double  range)
    /**********************************/
    {
	if (round_date_grid)    // Round at the date grids.
	{
	    return roundDateGrid (value, range);
	}
	else    // Round by interpolating the dates.
	{
	    return roundDate	 (value, range);
	}
	
    } // round ()


    /**
     * Return the rounded date value which is rounded at the date grids.
     *
     *    @param  value   The value to round.
     *
     *    @param  range   The rounding step may never exceeds this
     *			  range.
     *
     *    @return   The rounded value.
     *
    ********************************************/
    private double  roundDateGrid (double  value,
				   double  range)
    /*******************************************/
    {
	if (range == 0.0)    // Nothing to round: return.
	    return value;

	/*
	 * The rounding depends from the sign of the range: when it is
	 * positive (negative) we round to a larger (smaller) value.
	 */

	boolean  round_up = (range < 0.0 ? false : true);
	double	 step;

	range = Math.abs (range);  // Always use from now positive step.

	/*
	 * Round at the dates grid. A value at every 15 minutes is
	 * assumed!
	 */

	if (range < 1.0)    // < 15 m.: too small to round: return.
	    return value;

	if (range < 2.0)          // 15 m. - 30 m.:  1 grid step.
	    step = 1.0;
	else if (range < 4.0)	  // 30 m. - 60 m.:  2 grid steps.
	    step = 2.0;
	else if (range < 8.0)	  //  1 h. -  2 h.:  4 grid steps.
	    step = 4.0;
	else if (range < 12.0)    //  2 h. -  3 h.:  8 grid steps.
	    step = 8.0;
	else if (range < 24.0)    //  3 h. -  6 h.: 12 grid steps.
	    step = 12.0;
	else if (range < 48.0)    //  6 h. - 12 h.: 24 grid steps.
	    step = 24.0;
	else if (range < 96.0)	  // 12 h. - 24 h.: 48 grid steps.
	    step = 48.0;
	else			  // > 24 h.: use <n> * 96 grid steps.
	{
	    step = Math.floor (range / 96.0) * 96.0;
	}

	/*
	 * Round with the calculated step size.
	 */

	if (round_up)
	    value = Math.ceil  (value / step) * step;
	else
	    value = Math.floor (value / step) * step;

	return value;    // Return the rounded value.

    } // roundDateGrid ()


    /**
     * Return the rounded date value which is obtained by interpolating
     * between the date values.
     *
     *    @param  value   The value to round.
     *
     *    @param  range   The rounding step may never exceeds this
     *			  range.
     *
     *    @return   The rounded value.
     *
    ****************************************/
    private double  roundDate (double  value,
			       double  range)
    /***************************************/
    {
	if (range == 0.0)    // Nothing to round return.
	    return value;

	/*
	 * The rounding depends from the sign of the range: when it is
	 * positive (negative) we round to a larger (smaller) value.
	 */

	boolean  round_up = (range < 0.0 ? false : true);
	double	 step;

	range = Math.abs (range);  // Always use from now positive step.

	/*
	 * Round with integral values at the specified date values. A
	 * minute is the smallest step.
	 */

	if (range < MINUTES)    // < minute: too small to round: return.
	    return value;

	if (range < 2.0 * MINUTES)
	    step = MINUTES;
	else if (range < 5.0 * MINUTES)
	    step = 2.0 * MINUTES;
	else if (range < 10.0 * MINUTES)
	    step = 5.0 * MINUTES;
	else if (range < 15.0 * MINUTES)
	    step = 10.0 * MINUTES;
	else if (range < 30.0 * MINUTES)
	    step = 15.0 * MINUTES;
	else if (range < HOURS)
	    step = 30.0 * MINUTES;
	else if (range < 2.0 * HOURS)
	    step = HOURS;
	else if (range < 3.0 * HOURS)
	    step = 2.0 * HOURS;
	else if (range < 4.0 * HOURS)
	    step = 3.0 * HOURS;
	else if (range < 6.0 * HOURS)
	    step = 4.0 * HOURS;
	else if (range < 12.0 * HOURS)
	    step = 6.0 * HOURS;
	else if (range < 24.0 * HOURS)
	    step = 12.0 * HOURS;
	else			  // > 24 h.: round by <n> * 12 h.
	{
	    step = Math.floor (range / (24.0 * HOURS)) * (24.0 * HOURS);
	}

	/*
	 * Round with the calculates step size.
	 */

	if (round_up)
	    value = Math.ceil  (value / step) * step;
	else
	    value = Math.floor (value / step) * step;

	return value;    // Return the rounded value.

    } // roundDate ()


    /**
     * Return the next tick value, using the specified value and step.
     * This function uses the expectation that the values are integral
     * values. This function is part of the implementation of the
     * <CODE>DataHandler</CODE> interface for the dates axis.
     *
     *    @param  value   The value from which the next tick value must
     *			  be calculated.
     *
     *    @param  step	  The initial step to the next tick value.
     *
     *    @return   The next tick value.
     *
    *******************************************/
    public double  nextTickValue (double  value,
				  double  step)
    /******************************************/
    {
	if (value - Math.floor (value) > ROUND_BOUND)
	{
	    /*
	     * If the specified value within rounding errors is no
	     * integral value as expected, it is probably the first
	     * tick, so the next integral value is retuned.
	     */

	    return Math.ceil (value);
	}
	else if (step < 1.0)
	{
	    /*
	     * A step can't be smaller than an integral value, so it is
	     * corrected.
	     */

	    return value + 1.0;
	}
	else
	{
	    /*
	     * Return without changes the next tick value.
	     */

	    return value + step;
	}
    
    } // nextTickValue ()


    /**
     * Return the #&nbsp;tick label lines for these ticks. This function
     * is part of the implementation of the <CODE>DataHandler</CODE>
     * interface.
     *
     *    @return   The #&nbsp;tick label lines.
     *
    ****************************/
    public int  getNrTickLines ()
    /***************************/
    {
	return NR_DATE_TICK_LINES;
    }


    /**
     * Return the tick label lines specified by the current tick value
     * and tick step. This function is part of the implementation of the
     * <CODE>DataHandler</CODE> interface for the dates axis. Therefore
     * it depends from the type of date scaling.
     *
     *    @param  value   The tick value.
     *
     *    @param  step    The step to the next tick. It can be used to
     *			  adjust the value rounding when required.
     *
     *    @return   The tick label lines.
     *
    *********************************************/
    public String []  getTickLines (double  value,
				    double  step)
    /********************************************/
    {
	if (round_date_grid)    // Tick labels at the date grids.
	{
	    return getDateGridTickLines (value, step);
	}
	else    // Tick labels at the interpolation of dates.
	{
	    return getDateTickLines	(value, step);
	}

    } // getTickLines ()


    /**
     * Return the tick label lines specified by the current tick value
     * and tick step when the tick values are rounded at the date grid.
     *
     *    @param  value   The tick value.
     *
     *    @param  step	  The step to the next tick.
     *
     *    @return   The tick label lines.
     *
    ******************************************************/
    private String []  getDateGridTickLines (double  value,
					     double  step)
    /*****************************************************/
    {
	if (value - Math.floor (value) > ROUND_BOUND)    // No integral.
	{
	    /*
	     * Do nothing when the value is no integral.
	     */

	    date_marker [TIME_TICK_INDEX] = "";
	    date_marker [DATE_TICK_INDEX] = "";
	}
	else    // Integral value.
	{
	    /*
	     * Round into an integer index to avoid rounding errors.
	     */

	    int  index = (int) Math.round (value);

	    /*
	     * Set only the tick lables when the tick index fits within
	     * the date and time arrays.
	     */

	    if (index >= 0  &&  index < times.length)
	    {
		date_marker [TIME_TICK_INDEX] = times [index];
		date_marker [DATE_TICK_INDEX] = dates [index];
	    }
	    else
	    {
		date_marker [TIME_TICK_INDEX] = "";
		date_marker [DATE_TICK_INDEX] = "";
	    }

	} // Integral value.

	return date_marker;

    } // getDateGridTickLines ()


    /**
     * Return the tick label lines specified by the current tick value
     * and tick step when the dates are interpolated.
     *
     *    @param  value   The tick value.
     *
     *    @param  step	  The step to the next tick.
     *
     *    @return   The tick label lines.
     *
    **************************************************/
    private String []  getDateTickLines (double  value,
					 double  step)
    /*************************************************/
    {
	if (value - Math.floor (value) > ROUND_BOUND)    // No integral.
	{
	    /*
	     * Do nothing when the tick value is no integral.
	     */

	    date_marker [TIME_TICK_INDEX] = "";
	    date_marker [DATE_TICK_INDEX] = "";
	}
	else    // Integral value.
	{
	    /*
	     * Convert the relative tick time back to an absolute time.
	     * Round into a long to avoid rounding erros.
	     */

	    long  time = time_offset - Math.round (value);

	    /*
	     * Define the tick marker date with the specified time
	     * value.
	     */

	    marker_date_format.setTimeInMS (time);

	    /*
	     * Set the date and time label lines. Replace times by date
	     * IDs when a date ID table is specified and when a key can
	     * be found.
	     */

	    String  time_or_id;

	    if (date_id_table == null)    // Get short time.
	    {
		time_or_id = marker_date_format.getShortTime ();
	    }
	    else    // Get date ID.
	    {
		time_or_id = (String) date_id_table.get (
					  marker_date_format.getDate());

		/*
		 * Replace non-existing or empty date IDs by
		 * interopated values: The ID is the value of the next
		 * older date found.
		 */

		if (time_or_id == null  ||  time_or_id.equals (""))
		    time_or_id = interoplateDateIDFromMarker ();
	    }

	    date_marker [TIME_TICK_INDEX] = time_or_id;	   // Time.
	    date_marker [DATE_TICK_INDEX] =		   // Date.
		marker_date_format.getShortDate ();
	}

	return date_marker;    // Return the date tick label lines.

    } // getDateTickLines ()


    /**
     * Get the first date ID from the current marker date to the next
     * date in history. The empty string <CODE>&quot;&quot;</CODE> is
     * returned when no date ID was found.
     *
     *    @return  The next marker ID in history.
     *
    *********************************************/
    private String  interoplateDateIDFromMarker ()
    /********************************************/
    {
	String  date_id = null;    // Initially no date ID.

	int  date_vector_length = date_vector.size (); // Vector length.

	/*
	 * Set the current date format and index.
	 */

	int	    index = date_vector_index;
	DateFormat  date  = (DateFormat) date_vector.elementAt (index);

	/*
	 * When the current date is older than the marker date: go to
	 * the left (to the most youngest plot date). The last found
	 * date ID is then the correct one.
	 */

	if (date.compareTo (marker_date_format) <= 0)
	{
	    String  new_date_id;

	    do
	    {
		new_date_id = (String) date_id_table.get (
					   date.getDate ());

		if (new_date_id != null)
		    date_id = new_date_id;

		if (--index >= 0)
		    date = (DateFormat) date_vector.elementAt (index);
	    }
	    while ( index >= 0  &&
		    date.compareTo (marker_date_format) <= 0 );
	}

	/*
	 * Go to the right (to the history) from the current index when
	 * not yet a date ID was found.
	 */

	if (date_id == null)
	{
	    /*
	     * Go to the history from the current date as long as it is
	     * younger than the marker.
	     */

	    index = date_vector_index;

	    do
	    {
		if (++index < date_vector_length)
		    date = (DateFormat) date_vector.elementAt (index);
	    }
	    while ( index < date_vector_length  &&
		    date.compareTo (marker_date_format) > 0 );

	    /*
	     * Continue going to the history, but stop as soon as a
	     * date ID is found.
	     */

	    while (date_id == null  &&  index < date_vector_length)
	    {
		date_id = (String) date_id_table.get (date.getDate ());

		if (date_id == null  &&  ++index < date_vector_length)
		    date = (DateFormat) date_vector.elementAt (index);
	    }
	}

	/*
	 * Set the current date vector indices for the next run. Correct
	 * for crossing boundaries.
	 */

	if (index < 0)
	    date_vector_index = 0;
	else if (index >= date_vector_length)
	    date_vector_index = date_vector_length - 1;
	else
	    date_vector_index = index;

	/*
	 * Set the date ID to the empty string when it is undefined.
	 */

	if (date_id == null)
	    date_id = "";

	/*
	 * Return the date ID string.
	 */

	return date_id;

    } // interoplateDateIDFromMarker ()


    /**
     * Return if the plot frame is showed.
     *
     *    @return   The value of the show frame boolean.
     *
    ********************/
    boolean  getShowed ()
    /*******************/
    {
	/*
	 * Get the showed status synchronized with this class instance
	 * such that the plot frame will not be modified during this
	 * query.
	 */

	boolean  frame_showed;

	synchronized (this)
	{
	    frame_showed = ( plot_frame != null );
	}

	return frame_showed;
    }


    /**
     * Set the boundary which is used to define the foreground color,
     * depending from the green RGB value. The foreground is white when
     * the green RGB value is below this boundary; a black foreground is
     * used otherwise.
     *
     *    @param  green_white_fgnd_bnd   The green-white foreground
     *					 boundary.
    *****************************************************/
    void  setGreenWhiteFgndBnd (int  green_white_fgnd_bnd)
    /****************************************************/
    {
	this.green_white_fgnd_bnd = green_white_fgnd_bnd;
    }


    /**
     * Return the boundary which is used to define the foreground color,
     * depending from the green RGB value. The foreground is white when
     * the green RGB values are below this boundary; otherwise a black
     * background is used.
     *
     *    @return   The green-white foreground boundary.
     *
    ***************************/
    int  getGreenWhiteFgndBnd ()
    /**************************/
    {
	return green_white_fgnd_bnd;
    }


    /**
     * Set the legend background which is used when the corresponding
     * data are not plotted.
     *
     *    @param  no_plot_legend_bgnd   The specified legend
     *					background.
     *
    *****************************************************/
    void  setNoPlotLegendBgnd (Color  no_plot_legend_bgnd)
    /****************************************************/
    {
	this.no_plot_legend_bgnd = no_plot_legend_bgnd;
    }


    /**
     * Get the legend background which is used when the corresponding
     * data are not plotted.
     *
     *    @return   The legend background used when the data are not
     *		    plotted.
     *
    ****************************/
    Color  getNoPlotLegendBgnd ()
    /***************************/
    {
	return no_plot_legend_bgnd;
    }


    /**
     * Set the legend foreground which is used when the corresponding
     * data are not plotted.
     *
     *    @param  no_plot_legend_fgnd   The specified legend
     *					foreground.
     *
    *****************************************************/
    void  setNoPlotLegendFgnd (Color  no_plot_legend_fgnd)
    /****************************************************/
    {
	this.no_plot_legend_fgnd = no_plot_legend_fgnd;
    }


    /**
     * Get the legend foreground which is used when the corresponding
     * data are not plotted.
     *
     *    @return   The legend foreground used when the data are not
     *		    plotted.
     *
    ****************************/
    Color  getNoPlotLegendFgnd ()
    /***************************/
    {
	return no_plot_legend_fgnd;
    }


    /**
     * Set the plot title. It is the title of the plot frame.
     *
     *    @param  plot_title   The plot title.
     *
    **************************************/
    void  setPlotTitle (String  plot_title)
    /*************************************/
    {
	this.plot_title = plot_title;
    }


    /**
     * Set the label of the X axis.
     *
     *    @param  x_axis_label   The X axis label.
     *
    *****************************************/
    void  setXAxisLabel (String  x_axis_label)
    /****************************************/
    {
	this.x_axis_label = x_axis_label;
    }


    /**
     * Set the label of the Y axis.
     *
     *    @param  y_axis_label   The Y axis label.
     *
    *****************************************/
    void  setYAxisLabel (String  y_axis_label)
    /****************************************/
    {
	this.y_axis_label = y_axis_label;
    }


    /**
     * Set the hashtable with host keys to be excluded from the plot.
     *
     *    @param   no_plot_host_keys   The host keys to be excluded from
     *				       the plot.
     *
    **********************************************************/
    void  setNoPlotHostKeysTable (Hashtable  no_plot_host_keys)
    /*********************************************************/
    {
	/*
	 * Use buffer store to do not interfere with the current plot,
	 * if any.
	 */

	buffer_no_plot_host_keys = no_plot_host_keys;
    }


    /**
     * Set the dates used by the date grid rounding.
     *
     *    @param  dates   The pure date array for the date grid.
     *
     *    @param  times   The time array for the date grid.
     *
    ********************************/
    void  setDates (String []  dates,
		    String []  times)
    /*******************************/
    {
	/*
	 * Store the pure dates and times in buffer storage, because the
	 * current dates and times are part of the implementation of the
	 * "DataHandler" interface of this class instance. Therefore
	 * they may be used by the plot canvas.
	 */

	buffer_dates = dates;
	buffer_times = times;

    } // setDates ()


    /**
     * Set the time offset used by the dates interpolating.
     *
     *    @param  time_offset   The time offset.
     *
    **************************************/
    void  setTimeOffset (long  time_offset)
    /*************************************/
    {
	/*
	 * Store the time offset in a buffer, because the current time
	 * offset is part of the implementation of the "DataHandler"
	 * interface of this class instance. Therefore it may be used by
	 * the plot canvas.
	 */
	 
	buffer_time_offset = time_offset;

    } // setTimeOffset ()


    /**
     * Set the parameters used to obtain the date ID's. They are:
     *
     * <UL>
     *
     * <LI>An array with the used date ID's. They are used for
     *	   interpolation when no date ID was found in the date ID
     *	   hashtable.</LI>
     *
     * <LI>The date ID table. If this parameter is <CODE>null</CODE>,
     *	   no date ID table is used.</LI>
     *
     * <LI>The vector with date formats. If the date ID table is
     *	   <CODE>null</CODE>, it is unused, so in that case it may also
     *	   be <CODE>null</CODE>.</LI>
     *
     * </UL>
     *
     *    @param  date_id_table   The date ID table.
     *
     *    @param  date_vector	  The vector with date formats.
     *
    ****************************************************/
    void  setDateIDParameters (Hashtable   date_id_table,
			       Vector	   date_vector)
    /***************************************************/
    {
	/*
	 * Store the date ID table and the date formats in a buffer,
	 * because the current date ID table and date formats are part
	 * of the implementation of the "DataHandler" interface of this
	 * class instance. Therefore it may be used by the plot canvas.
	 */

	buffer_date_id_table = date_id_table;
	buffer_date_vector   = date_vector;

    } // setDateIDParameters ()


    /**
     * Set the external plot controls.
     *
     *    @param  extern_controls   The external plot controls.
     *
    ****************************************************************/
    void  setExternPlotControls (ExternPlotControls  extern_controls)
    /***************************************************************/
    {
	/*
	 * Store the external plot controls in a buffer, because the
	 * current external controls may be used by a redraw of the
	 * current plot frame.
	 */

	buffer_extern_controls = extern_controls;
    
    } // setExternPlotControls ()


    /**
     * Add a plot trace to the plot data storage. The plot trace vector
     * contains the points to plot which are instances of the
     * <CODE>DataPoint</CODE> class.
     *
     *    @param  trace      The plot trace to add.
     *
     *    @param  label	     The label of the plot trace. It is
     *			     displayed in the plot legend.
     *
     *    @param  host_key   The data key of the data, displayed with
     *			     the plot trace.
     *
    ********************************/
    void  addTrace (Vector  trace,
		    String  label,
		    String  host_key)
    /*******************************/
    {
	buffer_pot_plot_data.addElement (trace);
	buffer_plot_labels  .addElement (label);
	buffer_host_keys    .addElement (host_key);
    }


    /**
     * Change the plot outlook. Markers are added or removed. The plot
     * points are joined with line pieces or stay not connected. When
     * required the plot canvas is redraw.
     *
     *    @param  use_markers   (Do not) Mark the points with markers.
     *
     *    @param  use_join	(Do not) Join the plot points.
     *
    *****************************************************/
    private void  changePlotOutlook (boolean  use_markers,
				     boolean  use_join)
    /****************************************************/
    {
	/*
	 * Continue synchronized with this class instance such that the
	 * plot frame will not be modified during this by means of
	/*
	 * Always join points when no markers are used.
	 */

	if (!use_markers)
	    use_join = true;

	/*
	 * Set the class instance outlook parameters.
	 */

	this.use_markers = use_markers;
	this.use_join	 = use_join;

	/*
	 * Only continue when the plot frame is currently being
	 * displayed. Continue synchronized with this class instance
	 * such that the plot frame will not be modified during this
	 * operation.
	 */

	synchronized (this)    // For plot frame (not) shown status.
	{
	    /*
	     * Do nothing when the plot frame is currently not shown.
	     */

	    if (plot_frame == null)   // Plot frame is not shown.
	    {
		return;
	    }

	    /*
	     * Get the current plot outlook parameters from the plot
	     * canvas. They are needed to decide if the plot should be
	     * repainted.
	    */

	    boolean  plot_use_markers = plot.getUseMarkers ();
	    boolean  plot_use_join    = plot.getUseJoin    ();

	    if (use_markers == plot_use_markers)   // Unchanged markers.
	    {
		if (use_join != plot_use_join) // Changed point joining.
		{
		    /*
		     * It is sufficient to redisplay the control
		     * buttons, because their number and labels are
		     * changed and to replot the plot canvas with the
		     * changed outlook.
		     */

		    plot.quitPaint	(); // Quit the painting thread.
		    plot_frame.remove   (   // Remove the cntl panel; it
			control_panel);	    // will be recreated and ..
		    plot_frame.remove   (   // Remove the plot canvas to
			plot);		    // force a replot.
		    initNewControlPanel (); // ... the compon. are add.
		    setPlotOutlook	(); // Set outlook in plot canv.
		    addControlPanel	(); // Add the (new) cntl panel.
		    addPlotCanvas	(); // Add the plot canvas and
		    plot_frame.show	(); // show with changed comp.
		    plot_frame.repaint	(); // Repaint to force a replt.
		}
	    }
	    else    // Markers are changed.
	    {
		/*
		 * Remove all containers from the current plot frame and
		 * reshow the complete plot in the current frame,
		 * because all plot components are changed. Reason: the
		 * plot legend labels are changed: this implies that
		 * also the sizes are changed. Mark with the "false"
		 * argument in "showInPlotFrame()" that the window frame
		 * is currently being displayed.
		 */

		removeAllFromPlotFrame ();
		showInPlotFrame        (false);

	    } // Markers are changed.

	} // Synchronized this inst. for plot frame (not) shown status.
    
    } // changePlotOutlook ()


    /**
     * Show the plot. When it is already shown first remove all
     * components from the current frame and stop the plotting.
     *
    ****************/
    void  showPlot ()
    /***************/
    {
	/*
	 * (Re)show the plot synchronized with this class instance such
	 * that the plot frame will not be modified during this call.
	 */

	synchronized (this)    // For plot frame (not) show status.
	{
	    /*
	     * Initialise a new plot frame the plot frame is currently
	     * not shown. Otherwise remove all containers from the
	     * current plot frame. Store if a new plot frame should be
	     * enabled. This state should be supplied to the
	     * "showInPlotFrame()" function.
	     */

	    boolean  new_plot_frame;    // Store new plot frame state.

	    if (plot_frame == null)    // Plot frame is not shown.
	    {
		initNewPlotFrame ();

		new_plot_frame = true;
	    }
	    else    // Plot frame is shown.
	    {
		removeAllFromPlotFrame ();

		new_plot_frame = false;
	    }

	    /*
	     * Show the plot in the (new) plot frame. The required
	     * panel objects and corresponding layout objects are
	     * recreated and installed.
	     */

	    showInPlotFrame (new_plot_frame);

	} // Synchronized for plot frame (not) show status.

    } // showPlot ()


    /**
     * Show the plot in the current plot frame. The following steps are
     * taken to (re)show the plot:
     *
     * <UL>
     *	   <LI>
     *	   Resize the plot canvas when new sizes where specified with
     *	   <CODE>setNewSize()</CODE>.
     *	   </LI>
     *
     *	  <LI>
     *	  Set all data and parameters of the plot canvas.
     *	  </LI>
     *
     *	  <LI>
     *	  (Re)Show the plot.
     *	  </LI>
     * </UL>
     * </P>
     *
     * <P>
     * <EM>To be sure this method should be called synchronized such
     * that another thread can not change the status of the plot
     * frame, from or other plot dependent variables.</EM>
     *
     *    @param  new_plot_frame   This parameters should be
     *				   <CODE>true</CODE> when a new plot
     *				   frame is used. In that case the frame
     *				   should be packed. It is
     *				   <CODE>false</CODE> otherwise.
     *
     *    @see DataPlotWindow#setNewSize
     *
    ******************************************************/
    private void  showInPlotFrame (boolean  new_plot_frame)
    /*****************************************************/
    {
	/*
	 * Show the plot canvas with new sizes when they are set.
	 */

	if ( new_plot_width  > 0  &&
	     new_plot_height > 0 )      // New sizes are set.
	{
	    plot.resize (new_plot_width, new_plot_height);

	    new_plot_width  = 0;    // Set new width & height only once.
	    new_plot_height = 0;
	}

	/*
	 * Set the plot canvas parameters and data from the buffer
	 * storages when it is inactive.
	 */

	setNewExternControls ();    // Set the buffer external control.
	setNewPlotTitles     ();    // Set the buffer plot titles.
	setNewPlotData	     ();    // Set the buffer plot data.

	/*
	 * Initialise the components, when required, and add them to the
	 * new plot frame.
	 */

	initNewControlPanel ();   // Create new ctrl panel and add comp.
	setPlotOutlook	    ();   // Set outlook in plot canvas.
	addControlPanel     ();   // Add control panel to plot frame.

	if (use_legend_scrollbar)     // Add a legend scrollbar when ...
	    addLegendScrollbar ();    // ... the legend is too large.

	addLegend      ();   // Add the legend canvas to the plot frame.
	addPlotCanvas  ();   // Add the plot canvas.
	addStatusLabel ();   // Add the status label.

	/*
	 * Pack the plot frame when it is new and (re)show it.
	 */

	if (new_plot_frame)    // New plot frame.
	{
	    plot_frame.pack ();
	}

	plot_frame.show ();

    } // showInPlotFrame ()


    /**
     * Validate the plot window frame. Reshow the frame when the legend
     * scrollbar must be removed or add.
     * </P>
     *
     * <P>
     * <EM>This method is called from the corresponding method in the
     * plot frame class and is called synchronized with the plot
     * thread.</EM>
     *
     *    @see  DataPlotFrame#validate
     *
    ****************/
    void  validate ()
    /***************/
    {
	/*
	 * Check if the legend scrollbar must be added or removed.
	 * Reshow the plot when this is the case.
	 */

	if (!checkLegendScrollbar ())   // Change status scrollb. shown.
	{
	    showPlot ();
	}
    }


    /**
     * Get the label of the markers button. The markers state to set
     * with this button will be displayed.
     *
     *    @return   The label of the markers button.
     *
    ***************************************/
    private String  getMarkersButtonLabel ()
    /**************************************/
    {
	if (use_markers)
	    return NO_MARKERS_BUTTON_LABEL;
	else
	    return MARKERS_BUTTON_LABEL;
    }


    /**
     * Get the status message of the markers button.
     *
     *    @return   The status of the markers button.
     *
    ****************************************/
    private String  getMarkersButtonStatus ()
    /***************************************/
    {
	if (use_markers)
	    return NO_MARKERS_BUTTON_STATUS;
	else
	    return MARKERS_BUTTON_STATUS;
    }


    /**
     * Get the label of the join lines button. The join lines state to
     * set with this button will be displayed.
     *
     *    @return   The label of the join lines button.
     *
    ************************************/
    private String  getJoinButtonLabel ()
    /***********************************/
    {
	if (use_join)
	    return NO_JOIN_BUTTON_LABEL;
	else
	    return JOIN_BUTTON_LABEL;
    }


    /**
     * Get the status of the join lines button.
     *
     *    @return   The status message of the join lines button.
     *
    *************************************/
    private String  getJoinButtonStatus ()
    /************************************/
    {
	if (use_join)
	    return NO_JOIN_BUTTON_STATUS;
	else
	    return JOIN_BUTTON_STATUS;
    }


    /**
     * Check if the legend scrollbar must be added or removed. Return:
     *
     * <UL>
     *
     * <LI><CODE>true</CODE>: the scrollbar must be added or
     *     removed.</LI>
     *
     * <LI><CODE>false</CODE>: the scrollbar state is unchanged.</LI>
     *
     * </UL>
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the paint
     * thread.</EM>
     *
     *    @return   A boolean value which reflects a change in the state
     *		    of the legend scrollbar.
     *
    ***************************************/
    private boolean  checkLegendScrollbar ()
    /**************************************/
    {
	/*
	 * Obtain the legend size (only the height is important) from
	 * the scrollbar, or from the legend when the scrollbar is not
	 * used.
	 */

	Dimension  legend_size;

	if (use_legend_scrollbar)		    // Scrollbar used.
	    legend_size = legend_scrollbar.size ();
	else					    // Scrollbar unused.
	    legend_size = legend.size		();

	/*
	 * Inquire the minimum size from the legend which is required to
	 * show it completely.
	 */

	Dimension  min_legend_size = legend.getMinSize ();

	/*
	 * Store if the legend scrollbar is yet shown.
	 */

	boolean  old_use_legend_scrollbar = use_legend_scrollbar;

	/*
	 * Use the legend scrollbar if the current available height of
	 * the legend canvas is insufficient.
	 */

	if (legend_size.height < min_legend_size.height)
	    use_legend_scrollbar = true;
	else
	    use_legend_scrollbar = false;

	/*
	 * Mark a change in the using/unusing state of the legend
	 * scrollbar.
	 */

	boolean  use_unchanged =
		     (use_legend_scrollbar == old_use_legend_scrollbar);

	/*
	 * Adjust the legend scrollbar values when it is resized or
	 * removed. Also change the scroll offset of the legend.
	 */

	if (use_legend_scrollbar)    // Legend scrollbar is used.
	{
	    if (use_unchanged)    // The scrollbar was already used.
	    {
		/*
		 * Adjust the legend scrollbar values to a possible
		 * change in the height of the legend (scrollbar).
		 */

		legend_scrollbar.adjustValues (
				     legend_size.height,
				     min_legend_size.height);

		/*
		 * Set the scroll offset at the legend.
		 */

		legend.setYScroll (legend_scrollbar.getValue ());
	    }
	}
	else    // The legend scrollbar is unused.
	{
	    /*
	     * Reset the legend scrollbar and legend canvas to the
	     * initialisation values.
	     */

	    legend_scrollbar.initParameters ();
	    legend.setYScroll		    (0);
	}

	return use_unchanged; // Return a change in the scrollbar state.

    } // checkLegendScrollbar ()


    /**
     * Handle an event in the plot frame. If a component is found to
     * handle this event, <CODE>true</CODE> is returned. If no such
     * component is found <CODE>false</CODE> is returned such that the
     * corresponding method in the <CODE>DataPlotFrame</CODE> class
     * instance can call its super class method to return the
     * appropriate result.
     * </P>
     *
     * <P>
     * <EM>This method is called from the corresponding method in the
     * plot frame class and is called synchronized with the plot
     * thread.</EM>
     *
     *    @param  event   The event to handle.
     *
     *    @return   A boolean which reflects if the event could be
     *		    handled.
     *
     *	  @see  DataPlotFrame#handleEvent
     *
    **********************************/
    boolean  handleEvent (Event  event)
    /*********************************/
    {
	switch (event.id)    // Analyse the event type.
	{
	    case Event.SCROLL_ABSOLUTE  :    // Handle a scroll event.
	    case Event.SCROLL_LINE_DOWN :
	    case Event.SCROLL_LINE_UP   :
	    case Event.SCROLL_PAGE_DOWN :
	    case Event.SCROLL_PAGE_UP   :

		/*
		 * Handle the scroll event when it occurred in the
		 * legend scrollbar.
		 */

		if (event.target == legend_scrollbar)
		{
		    handleScroll ();	// Handle legend scroll event.
		    return true;	// Scroll event handled.
		}
		break;

	    case Event.WINDOW_DESTROY :    // A window is destroyed.

		/*
		 * Stop the plot, remove all component and dispose this
		 * plot frame when this frame is destroyed.
		 */

		if (event.target == plot_frame)
		{
		    disposePlot ();    // Dispose the plot frame.
		    return true;       // Destroy event handled.
		}
		break;

	    default :    // Ignore in this function other events.

		break;
	}

	/*
	 * Let other events and/or targets handle by the corresponding
	 * super method from the plot frame class instance that called
	 * this function.
	 */

	return false;

    } // handleEvent ()


    /**
     * Handle scrolling in the legend scrollbar.
     *
    ****************************/
    private void  handleScroll ()
    /***************************/
    {
	/*
	 * Let the legend scrollbar store its values. They may be
	 * needed to adjust the scrollbar after a resize.
	 */

	legend_scrollbar.storeParameters ();

	/*
	 * Set the new scroll offset of the legend canvas and repaint
	 * it.
	 */

	legend.setYScroll (legend_scrollbar.getValue ());
	legend.repaint	  ();

    } // handleScroll ()


    /**
     * Handle an action event in the plot frame. If a component is found
     * to handle this event, <CODE>true</CODE> is returned. If no such
     * component is found <CODE>false</CODE> is returned such that the
     * corresponding method in the <CODE>DataPlotFrame</CODE> class
     * instance can call its super class method to return the
     * appropriate result.
     * </P>
     *
     * <P>
     * <EM>This method is called from the corresponding method in the
     * plot frame class and is called synchronized with the plot
     * thread.</EM>
     *
     *    @param  event    The action event to handle.
     *
     *    @param  object   The subject of the action event, often it is
     *			   <CODE>Event.arg</CODE>.
     *
     *    @return   A boolean which reflects if the action event could
     *		    be handled.
     *
     *	  @see  DataPlotFrame#action
     *
    *******************************/
    boolean  action (Event   event,
		     Object  object)
    /******************************/
    {
	/*
	 * Analyse the target of the action event and take the
	 * appropriate steps.
	 */

	if (event.target == close_button)    // Frame should be closed.
	{
	    disposePlot ();    // Remove components and close the frame.
	    return true;
	}
	else if (event.target == markers_button)    // Yes/No markers.
	{
	    /*
	     * Change the plot outlook after toggling the marker usage
	     * flag and reshow the plot.
	     */

	    changePlotOutlook ( !plot.getUseMarkers (),
			         plot.getUseJoin    () );
	    return true;
	}
	else if (event.target == join_button)    // Yes/No join lines.
	{
	    /*
	     * Change the plot outlook after toggling the joining of
	     * lines and reshow the plot.
	     */

	    changePlotOutlook (  plot.getUseMarkers (),
				!plot.getUseJoin    () );
	    return true;
	}
	else if ( extern_controls != null  &&
		  extern_controls.plotControlAction (event, object) )
	{
	    /*
	     * An external control action occurred and is handled by the
	     * external control interface: no more to be done with this
	     * action event.
	     */

	    return true;
	}
	else    // The target is not found here.
	{
	    /*
	     * Let other events and/or targets handle by the
	     * corresponding super method from the plot frame class
	     * instance that called this function.
	     */

	    return false;
	}

    } // action ()

    /**
     * Dispose the plot window, when showm. First remove all components
     * from the current plot frame and clear the plot frame, the layout
     * and the used panel objects.
     *
    *******************/
    void  disposePlot ()
    /******************/
    {
	/*
	 * Dispose the plot synchronized with this class instance such
	 * that no other thread can change the status of the plot frame.
	 */

	synchronized (this)    // For plot frame (not) show status.
	{
	    /*
	     * Only dispose the plot frame when it is currently being
	     * shown.
	     */

	    if (plot_frame != null)    // Plot frame is currently shown.
	    {
		plot_frame.dispose     ();    // Dispose plot frame.
		removeAllFromPlotFrame ();    // Remove containers.
		clearPlotFrameVar      ();    // Clear plot frame param.
	    }

	} // Synchronized this class instance for plot frame.

    } // disposePlot ()


    /**
     * Remove all components from the plot frame. First quit the paint
     * thread.
     * </P>
     *
     * <P>
     * <EM>To be sure this method should be called synchronized such
     * that another thread can not change the status of the plot
     * frame.</EM>
     *
    **************************************/
    private void  removeAllFromPlotFrame ()
    /*************************************/
    {
	plot.quitPaint	     ();     // Quit the plot painting thread.
	plot_frame.removeAll ();     // Remove all comp. from the frame.
    }


    /**
     * Create a new instance of the plot frame. Initialise it also with
     * its corresponding default settings. Create also a new GridBag
     * layout and install it as layout manager.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized.</EM>
     *
    ********************************/
    private void  initNewPlotFrame ()
    /*******************************/
    {
	/*
	 * Create a new plot frame class instance and initialise it to
	 * the corresponding default settings. This class instance is
	 * supplied to it to be able to handle the appropriate instances
	 * by this class.
	 */

	plot_frame = new DataPlotFrame (this);

	/*
	 * Initialise below the plot frame to its default settings.
	 */

	/*
	 * Set the default font used in the plot window.
	 */

	plot_frame.setFont (FONT);

	/*
	 * Set the default window background, equal to that of the plot
	 * canvas (with the exception of the drawing area).
	 */

	plot_frame.setBackground (BACKGROUND);

	/*
	 * Create the plot frame GridBag layout and install it as its
	 * layout manager.
	 */

	grid_bag = new GridBagLayout ();

	plot_frame.setLayout (grid_bag);

    } // initNewPlotFrame ()


    /**
     * Clear the plot frame and related layout and panel class
     * variables.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    *********************************/
    private void  clearPlotFrameVar ()
    /********************************/
    {
	plot_frame	 = null;    // Undefine the plot frame and ...
	grid_bag	 = null;    // ... its layout.
	control_panel	 = null;    // Undefine the control panel ...
	control_grid_bag = null;    // ... and its layout.
    }


    /**
     * Set the plot title and axis labels, stored in the window frame,
     * to the plot canvas.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    ********************************/
    private void  setNewPlotTitles ()
    /*******************************/
    {
	plot_frame.setTitle (plot_title);
	plot.setXAxisLabel  (x_axis_label);
	plot.setYAxisLabel  (y_axis_label);
    }


    /**
     * Set the plot data and the date ticks rounding storage, stored in
     * the window frame, to the plot canvas. Exclude traces from which
     * the host keys are in the excluded host keys.
     *
    ******************************/
    private void  setNewPlotData ()
    /*****************************/
    {
	/*
	 * Do nothing when no date tick rounding information (from both
	 * types) or plot data are available.
	 */

	if ( round_date_grid  && 
	     (buffer_dates == null || buffer_times == null)
	     ||
	     !round_date_grid  &&  buffer_time_offset < 0
	     ||
	     buffer_pot_plot_data.size () <= 0 )
	{
	    return;
	}

	/*
	 * Add the non-excluded plot traces and legends.
	 */

	plot_labels	  = buffer_plot_labels;
	pot_plot_data	  = buffer_pot_plot_data;
	host_keys	  = buffer_host_keys;
	no_plot_host_keys = buffer_no_plot_host_keys;

	/*
	 * Set the plot data traces and legends to show to the plot.
	 */

	setPlotDataAndLegendsShown ();

	/*
	 * Set the date ticks roundings for both methods to the plot
	 * canvas.
	 */

	dates	          = buffer_dates;
	times	          = buffer_times;
	time_offset       = buffer_time_offset;
	date_id_table     = buffer_date_id_table;
	date_vector       = buffer_date_vector;
	date_vector_index = 0;

	/*
	 * Re-initialise the storage variables.
	 */

	buffer_host_keys	 = new Vector ();
	buffer_no_plot_host_keys = null;
	buffer_plot_labels	 = new Vector ();
	buffer_pot_plot_data	 = new Vector ();
	buffer_dates		 = null;
	buffer_times		 = null;
	buffer_date_id_table	 = null;
	buffer_date_vector	 = null;
    
    } // setNewPlotData


    /**
     * Set the plot data traces and legends to show to the plot.
     *
    ******************************************/
    private void  setPlotDataAndLegendsShown ()
    /*****************************************/
    {
	int  nr_pot_plot_data = pot_plot_data.size ();

	Vector  real_plot_data   = new Vector (nr_pot_plot_data);
	Vector  real_plot_labels = new Vector (nr_pot_plot_data);
	int	data_index	 = 0;
	int	legend_index	 = 0;
	int	trace_index	 = 0;

	/*
	 * Reallocate translation arrays when the plot size is changed.
	 */
	 
	if ( legend_to_data_index	 == null  ||
	     legend_to_data_index.length != nr_pot_plot_data )
	{
	    legend_to_data_index  = new int [nr_pot_plot_data];
	    legend_to_trace_index = new int [nr_pot_plot_data];
	    trace_to_legend_index = new int [nr_pot_plot_data];
	}

	/*
	 * Add the not excluded traces. Add the legends corresponding to
	 * the flag if all legends are shown, or only the included ones.
	 * Store the transformation from legend to real legend index in
	 * the plot.
	 */
	 
	while (data_index < nr_pot_plot_data)    // Do for all data.
	{
	    String   host_key	= (String) host_keys.elementAt (
							 data_index);
	    boolean  plot_trace = !no_plot_host_keys.containsKey (
							 host_key);

	    /*
	     * Add the plot traces shown.
	     */
	     
	    if (plot_trace)
		real_plot_data.addElement (pot_plot_data.elementAt (
					       data_index));

	    /*
	     * Add the legends of the corresponding plot traces shown.
	     * Add also the legends of the plot traces not shown, when
	     * selected. Set the concersion tables from legend index to
	     * data index and to trace index, if any.
	     */

	    if (plot_trace || show_no_plot_legends)
	    {
		/*
		 * Add legend label of (not) shown data.
		 */
		 
		real_plot_labels.addElement (plot_labels.elementAt (
					         data_index));

		/*
		 * Connect legend to data and trace to legend.
		 */
		 
		legend_to_data_index [legend_index] = data_index;

		if (plot_trace)    // Connect to plot trace.
		{
		    legend_to_trace_index [legend_index] = trace_index;
		    trace_to_legend_index [trace_index ] = legend_index;

		    trace_index++;
		}
		else    // No plot trace connected with legend.
		{
		    legend_to_trace_index [legend_index] = -1;
		}

		legend_index++;

	    } // Add legend.

	    data_index++;

	} // Do for all data.

	/*
	 * Plot at least the first plot trace when none are defined.
	 */

	if (legend_index == 0)    // No plot traces defined.
	{
	    real_plot_data.addElement   (pot_plot_data.elementAt (0));
	    real_plot_labels.addElement (plot_labels.elementAt   (0));

	    legend_to_data_index  [0] = 0;
	    legend_to_trace_index [0] = 0;
	}

	/*
	 * Set the legend labels to the plot canvas.
	 */

	legend.setLabels (real_plot_labels);

	/*
	 * Set the data trace vectors to the plot canvas.
	 */

	plot.setDataVector (real_plot_data);
	plot.unZoom	   ();

    } // setPlotDataShownAndLegends ()


    /**
     * Create a new instance of the control panel. Create also a new
     * instance of its GridBag layout manager and install it as such.
     * The components are add to the control panel in correspondence
     * with the current plot outlook.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    ***********************************/
    private void  initNewControlPanel ()
    /**********************************/
    {
	/*
	 * Create new instances of the control panel and its GridBag
	 * layout which is installed as layout manager.
	 */

	control_panel	 = new Panel	     ();
	control_grid_bag = new GridBagLayout ();

	control_panel.setLayout (control_grid_bag);

	/*
	 * Add the close button to the control panel, using left
	 * alignment in the gridbag (2th "false" function argument).
	 */

	addToControlPanel (close_button, false);

	/*
	 * Add the marker button with the appropriate button label,
	 * depending from the current use (no) marker state. Right
	 * alignment is used (2th "true" function argument.
	 */

	markers_button.setLabel (getMarkersButtonLabel ());
	addToControlPanel	(markers_button, true);

	/*
	 * Add only the join button when markers are shown. Change the
	 * appropriate button label depending from the current yes/no
	 * join points state.
	 */

	if (use_markers)    // Markers are shown.
	{
	    join_button.setLabel (getJoinButtonLabel ());
	    addToControlPanel    (join_button, true);
	}

	/*
	 * Add the extern controls to the control panel.
	 */

	addExternControls ();

    } // initNewControlPanel ()


    /**
     * Set the plot outlook in the plot canvas.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    ******************************/
    private void  setPlotOutlook ()
    /*****************************/
    {
	plot.setUseMarkers (use_markers);
	plot.setUseJoin	   (use_join);
    }


    /**
     * Set the new extern controls from the storage. They will be used
     * in the control panel with the "ExternPlotControls" interface.
     *
    ************************************/
    private void  setNewExternControls ()
    /***********************************/
    {
	extern_controls = buffer_extern_controls;
    }


    /**
     * Add the extern controls to the control panel. Before they are
     * added let the instance of the "ExternPlotControls" adjust these
     * controls.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    *********************************/
    private void  addExternControls ()
    /********************************/
    {
	/*
	 * Do nothing when no extern controls are defined.
	 */

	if (extern_controls == null)
	    return;

	/*
	 * The extern controls are adjusted by the interface.
	 */

	extern_controls.adjustPlotControls ();

	/*
	 * Obtain the extern controls from the interface.
	 */

	Component []  controls    = extern_controls.getPlotControls ();
	int	      nr_controls = controls.length;

	/*
	 * Add the extern controls to the control panel.
	 */

	for (int index = 0;  index < nr_controls;  index++)
	    addToControlPanel (controls [index], true);

    } // addExternControls ()


    /**
     * Add a control component to the control panel.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
     *    @param  control_component   The component to add.
     *
     *    @param  use_left_border     <CODE>true</CODE> add left border;
     *				      <CODE>false</CODE> otherwise.
     *
    *************************************************************/
    private void  addToControlPanel (Component  control_component,
				     boolean	use_left_border)
    /************************************************************/
    {
	/*
	 * Create new constraints for the control panel.
	 */

	GridBagConstraints  constraints = new GridBagConstraints ();

	/*
	 * Set the constraints fields for the control panel. The
	 * components are placed at a row with the specified horizontal
	 * gap at the right border, when specified.
	 */

	int  left_border = (use_left_border ? INTER_BORDER : 0);

	constraints.gridy  = 0;
	constraints.insets = new Insets (0, left_border, 0, 0);

	control_grid_bag.setConstraints (control_component,
					 constraints);
	control_panel.add		(control_component);

    } // addToControlPanel ();


    /**
     * Add the control panel to the window frame, using the appropriate
     * <CODE>GridBagConstraints</CODE>.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    *******************************/
    private void  addControlPanel ()
    /******************************/
    {
	/*
	 * Allocate the grid bag constraints for the control panel.
	 */

	GridBagConstraints  constraints = new GridBagConstraints ();

	/*
	 * Set the grid bag grid width: the control panel spans over the
	 * complete plot window frame.
	 */

	int  gridwidth;

	if (use_legend_scrollbar)
	    gridwidth = 3;
	else
	    gridwidth = 2;

	/*
	 * Define the "Insets" for the borders around the control panel.
	 */

	Insets  insets = new Insets (PLOT_BORDER,     // Top.
				     PLOT_BORDER,     // Left.
				     AREA_BORDER,     // Bottom.
				     PLOT_BORDER);    // Right.

	/*
	 * Set the constraints fields.
	 */
	 
	constraints.gridx     = 0;   // Control panel is at the top row.
	constraints.gridy     = 0;
	constraints.gridwidth = gridwidth;
	constraints.insets    = insets;

	/*
	 * Set the control panel constraints and add it to the window
	 * frame.
	 */


	grid_bag.setConstraints (control_panel, constraints);
	plot_frame.add		(control_panel);

    } // addControlPanel ()


    /**
     * Add the legend scrollbar to the window frame, using the
     * appropriate <CODE>GridBagConstraints</CODE>.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    **********************************/
    private void  addLegendScrollbar ()
    /*********************************/
    {
	GridBagConstraints  constraints = new GridBagConstraints ();

	/*
	 * Define the borders in the plot frame. Note that the border
	 * at the top is defined by the adjacent control panel. No
	 * border is let to the right with the legend.
	 */

	Insets  insets = new Insets (0,		     // Top.
				     PLOT_BORDER,    // Left.
				     AREA_BORDER,    // Bottom.
				     0);	     // Right.

	/*
	 * Set the constraints fields.
	 */

	constraints.fill       = GridBagConstraints.VERTICAL;
	constraints.gridx      = 0;
	constraints.gridy      = 1;
	constraints.weightx    = 0.0;    // No horizontal fill.
	constraints.weighty    = 1.0;    // Full vertical fill.
	constraints.gridheight = 2;	 // Next to canvas + status lab.
	constraints.insets     = insets;

	/*
	 * Set the legend scrollbar constraints and add it to the window
	 * frame.
	 */

	grid_bag.setConstraints (legend_scrollbar, constraints);
	plot_frame.add		(legend_scrollbar);

    } // addLegendScrollbar ()


    /**
     * Add the plot label legend to the plot frame, using the
     * appropriate <CODE>GridBagConstraints</CODE>.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    *************************/
    private void  addLegend ()
    /************************/
    {
	/*
	 * Set the initial sizes of the legend and resize it to its
	 * minimal required size.
	 */

	legend.initSizes ();
	legend.resize	 (legend.getMinSize ());

	GridBagConstraints  constraints = new GridBagConstraints ();

	int     x_plot_border;
	int     fill;
	int     gridx;
	double  weighty;

	if (use_legend_scrollbar)    // A legend scrollbar is used.
	{
	    /*
	     * Fill the legend vertically to the available space. Let no
	     * border between legend and legend scrollbar.
	     */

	    x_plot_border = 0;
	    fill	  = GridBagConstraints.VERTICAL;
	    gridx	  = 1;
	    weighty	  = 1.0;
	}
	else
	{
	    /*
	     * Do not fill the legend when it fits into the plot frame.
	     * The border at the left is the usual plot border.
	     */

	    x_plot_border = PLOT_BORDER;
	    fill	  = GridBagConstraints.NONE;
	    gridx	  = 0;
	    weighty	  = 0;
	}

	/*
	 * Define the borders in the plot frame. The border at the top
	 * is defined by the adjacent control panel.
	 */

	Insets  insets = new Insets (0,		       // Top.
				     x_plot_border,    // Left.
				     AREA_BORDER,      // Bottom.
				     AREA_BORDER);     // Right.

	/*
	 * Set the constraints fields.
	 */

	constraints.anchor     = GridBagConstraints.NORTHWEST;
	constraints.fill       = fill;
	constraints.gridx      = gridx;
	constraints.gridy      = 1;
	constraints.weightx    = 0.0;
	constraints.weighty    = weighty;
	constraints.gridheight = 2;	  // Span over canvas + stat. l.
	constraints.insets     = insets;

	/*
	 * Set the legend constraints and add it to the plot frame.
	 */

	grid_bag.setConstraints (legend, constraints);
	plot_frame.add		(legend);

    } // addLegend ()


    /**
     * Add the plot canvas to the plot frame, using the appropriate
     * <CODE>GridBagConstraints</CODE>.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    *****************************/
    private void  addPlotCanvas ()
    /****************************/
    {
	GridBagConstraints  constraints = new GridBagConstraints ();

	int  gridx;

	if (use_legend_scrollbar)
	    gridx = 2;
	else
	    gridx = 1;

	/*
	 * Define the borders in the plot frame. The border at the top
	 * (left) is defined by the adjacent control panel (legend).
	 */

	Insets  insets = new Insets (0,		      // Top.
				     0,		      // Left.
				     AREA_BORDER,     // Bottom.
				     PLOT_BORDER);    // Right.

	/*
	 * Set the constraints fields.
	 */

	constraints.fill   = GridBagConstraints.BOTH;
	constraints.gridx   = gridx;
	constraints.gridy   = 1;
	constraints.insets  = insets;
	constraints.weightx = 1.0;
	constraints.weighty = 1.0;

	/*
	 * Set the plot canvas constraints and add it to the plot frame.
	 */

	grid_bag.setConstraints (plot, constraints);
	plot_frame.add		(plot);

    } // addPlotCanvas ()


    /**
     * Add the status label to the plot frame, using the appropriate
     * <CODE>GridBagConstraints</CODE>.
     * </P>
     *
     * <P>
     * <EM>This method should be called synchronized with the painting
     * thread.</EM>
     *
    ******************************/
    private void  addStatusLabel ()
    /*****************************/
    {
	GridBagConstraints  constraints = new GridBagConstraints ();

	int  gridx;

	if (use_legend_scrollbar)
	    gridx = 2;
	else
	    gridx = 1;

	/*
	 * Define the borders in the plot frame. The border at the top
	 * (left) is defined by the adjacent plot canvas (legend).
	 */

	Insets  insets = new Insets (0,		      // Top.
				     0,		      // Left.
				     PLOT_BORDER,     // Bottom.
				     PLOT_BORDER);    // Right.

	/*
	 * Set the constraints fields.
	 */
	 
	constraints.fill   = GridBagConstraints.BOTH;
	constraints.gridx  = gridx;
	constraints.gridy  = 2;
	constraints.insets = insets;

	/*
	 * Set the status label constraints and add it to the plot
	 * frame.
	 */

	grid_bag.setConstraints (status_label, constraints);
	plot_frame.add		(status_label);

    } // addStatusLabel ()

} // class DataPlotWindow
