/* ------------------------------------------------------------------ */
/*		     G e n H o s t K e y . j a v a		      */
/*		     =============================		      */
/*								      */
/*   This file contains the source of the "GenHostKey" class.	      */
/* ------------------------------------------------------------------ */


/**
 * This class is used by generating a host key from the specified
 * host(s) and the net type. When more hosts are specified they may be
 * sorted before generating the host key. This option is set at the
 * constructor.
 *
 *    @see GenHostKey#GenHostKey
 *
 *    @author Hans Blom
 *
****************/
class  GenHostKey
/***************/
{
    /*
     * Default do not sort hosts.
     */

    private static final boolean  DEFAULT_SORT_HOSTS = false;

    /*
     * Define the separator used in the host(s) key. It is used between
     * two host names.
     */

    private static final char  KEY_SEPAR = '-';

    /*
     * Set the class instance variables.
     */
     
    private boolean  sort_hosts;    // Flag to sort the specified hosts.


    /**
     * Constructor to initialise this class.
     *
    ************/
    GenHostKey ()
    /***********/
    {
	sort_hosts = DEFAULT_SORT_HOSTS;
    }


    /**
     * Set if the hosts should be sorted.
     *
     *    @param  sort_hosts   Sort the hosts from which the key must be
     *			       generated.
     *
    **************************************/
    void setSortHosts (boolean  sort_hosts)
    /*************************************/
    {
	this.sort_hosts = sort_hosts;
    }


    /**
     * Get the value of the flag which determines if the hosts should be
     * sorted.
     *
     *    @return  Return <CODE>true</CODE> when the hosts should be
     *		   sorted; <CODE>false</CODE> otherwise.
     *
    ***********************/
    boolean  getSortHosts ()
    /**********************/
    {
	return sort_hosts;
    }


    /**
     * Return the host key which is connected with the hosts in the
     * specified array and with the specified net type. when selected
     * from the constructor and when more than one host is specified:
     * the hosts array will be sorted by this function in alphabetical
     * order prior to the key determination, so if sorting is selected,
     * the hosts array storage is <EM>not</EM> constant.
     *
     *    @param  hosts      The array specifying the hosts.
     *
     *    @param  net_type   The QoS of Inet net type.
     *
     *    @return  The host key.
     *
     *    @see  GenHostKey#GenHostKey
     *
    ********************************/
    String  get (String []  hosts,
		 String     net_type)
    /*******************************/
    {
	/*
	 * Sort the hosts array when selected and when more than one
	 * host is specified.
	 */

	if (sort_hosts  &&  hosts.length > 1)
	    sort (hosts);

	/*
	 * Allocate the buffer which is used to store the host key.
	 */

	StringBuffer  host_key_sb = new StringBuffer ();

	/*
	 * Set the host key. It is cnstructed by placing the
	 * "Host_SEPAR" bewteen two successive host names.
	 */

	int  nr_hosts = hosts.length;
	int  i;

	for (i = 0;  i < nr_hosts;  i++)
	{
	    host_key_sb.append (hosts [i]);
	    host_key_sb.append (KEY_SEPAR);
	}

	host_key_sb.append (net_type);

	return host_key_sb.toString ();   // Return the host key as str.

    } // get ()


    /**
     * Sort a String array. Because the arrays are small a very simple
     * sort routine is used.
     *
     *    @param  ar  The string array to sort.
     *
    ****************************************/
    private static void  sort (String []  ar)
    /***************************************/
    {
	int  max_ind  = ar.length - 1;

	/*
	 * Sort the elements, starting at the first index.
	 */

	for (int sort_ind = 0;  sort_ind < max_ind;  sort_ind++)
	{
	    /*
	     * Sort the elements at indices larger than the index of the
	     * last sorted element. Note that the other elements are
	     * guaranteed sorted.
	     */

	    int	    min_ind = sort_ind;
	    String  min_str = ar [min_ind];

	    for (int ind = min_ind + 1;  ind <= max_ind;  ind++)
		if (min_str.compareTo (ar [ind]) > 0)    // New minimum.
		{
		    min_ind = ind;
		    min_str = ar [min_ind];
		}

	    /*
	     * Swap between minimum and position to sort when these
	     * elements are not equal.
	     */

	    if (min_ind != sort_ind)
	    {
		ar [min_ind ] = ar [sort_ind];
		ar [sort_ind] = min_str;
	    }

	} // Sort the elements.

    } // sort ()

} // class GenHostKey
