/* ------------------------------------------------------------------ */
/*		   L e g e n d C a n v a s . j a v a		      */
/*		   =================================		      */
/*								      */
/*   This file contains the source of the "LegendCanvas" class.       */
/* ------------------------------------------------------------------ */


import java.util.*;
import java.awt.*;
import user.plot.*;


/**
 * This class is used to write the legends of the plot traces in a
 * Canvas. The background of a legend has the same color as the markers
 * and/or line pieces of the corresponding plot trace. When the trace
 * points are marked with markers the corresponding markers are also
 * drawn before the trace labels.
 *
 *    @author  Hans Blom
 *
***********************************/
class  LegendCanvas  extends  Canvas
/**********************************/
{
    /*
     * Define a percentage used to calculate the spacing fraction at the
     * top from the descent between the baseline of a fonts characters
     * and its bottom.
     */

    private static final int TOP_DESCENT_PERC = 50; // %

    /*
     * Define the default border, the separation between two rows and
     * the separation between marker and label.
     */

    private static final int  DEFAULT_BORDER		 = 2;
    private static final int  DEFAULT_ROW_SEPAR		 = 2;
    private static final int  DEFAULT_MARKER_LABEL_SEPAR = 6;

    /*
     * Set the class instance variables.
     */

    private PlotLegendHandler  legend_handler;	   // Plot leg. handler.
    private Font	       font;		   // Legend font.
    private FontMetrics	       font_metrics;	   // Leg. font metrics.
    private Vector             labels;		   // Plot labels.
    private Dimension          min_size;	   // Min. legend sizes.
    private Dimension	       legend_size;	   // Legend sizes.
    private int		       y_scroll;	   // Y scroll position.
    private int		       y_font_offset;	   // Y offset top font.
    private int		       font_height;	   // The font height;
    private int		       row_height;	   // Height legend row.
    private int		       border;		   // Legend border.
    private int		       row_separ;	   // Separ. between r.
    private int		       marker_label_separ; // Separ. marker-lab.


    /**
     * Constructor.
     *
     *    @param  legend_handler   The plot legend handler handling
     *				   aspects connected with the plot to
     *				   which this legend belongs.
     *
    ***********************************************/
    LegendCanvas (PlotLegendHandler  legend_handler)
    /**********************************************/
    {
	/*
	 * Initialise the class instances.
	 */

	this.legend_handler	= legend_handler;
	this.font		= null;
	this.font_metrics	= null;
	this.labels		= null;
	this.min_size		= new Dimension (0, 0);
	this.legend_size	= new Dimension (0, 0);
	this.y_scroll		= 0;
	this.y_font_offset	= 0;
	this.font_height	= 0;
	this.row_height		= 0;
	this.border		= DEFAULT_BORDER;
	this.row_separ		= DEFAULT_ROW_SEPAR;
	this.marker_label_separ = DEFAULT_MARKER_LABEL_SEPAR;

    } // LegendCanvas ()


    /**
     * Set the plot labels.
     *
     *    @param  labels   The plot labels.
     *
    *******************************/
    void  setLabels (Vector  labels)
    /******************************/
    {
	this.labels = labels;
    }


    /**
     * Get the plot labels.
     *
     *    @return   The plot labels.
     *
    *******************/
    Vector  getLabels ()
    /******************/
    {
	return labels;
    }


    /**
     * Get the plot label size.
     *
     *    @return   The plot label size.
     *
    *******************/
    int  getLabelSize ()
    /******************/
    {
	return labels.size ();
    }


    /**
     * Set the border around the legend labels.
     *
     *    @param  border   The border.
     *
    ****************************/
    void  setBorder (int  border)
    /***************************/
    {
	this.border = border;
    }


    /**
     * Get the border around the legend labels.
     *
     *    @return   The border.
     *
    ****************/
    int  getBorder ()
    /***************/
    {
	return border;
    }


    /**
     * Set the separator between marker and label.
     *
     *    @param  marker_label_separator   The separator.
     *
    **************************************************/
    void  setMarkerLabelSepar (int  marker_label_separ)
    /*************************************************/
    {
	this.marker_label_separ = marker_label_separ;
    }


    /**
     * Get the separator between marker and label.
     *
     *    @return   The separator.
     *
    **************************/
    int  getMarkerLabelSepar ()
    /*************************/
    {
	return marker_label_separ;
    }


    /**
     * Initialise the sizes of the legend canvas. They depend from the
     * Used font and the specified labels.
     *
    *****************/
    void  initSizes ()
    /****************/
    {
	/*
	 * Get the font characteristics.
	 */

	font	     = getFont	      ();
	font_metrics = getFontMetrics (font);

	/*
	 * Set the height of the font. No inter-line spacing is used,
	 * but a border at the top and bottom of a legend is used
	 * instead. Minimal the font height is the sum of the maximum
	 * ascent above the font baseline and the minimum descent below
	 * the baseline
	 */

	font_height = font_metrics.getMaxAscent  () +
		      font_metrics.getMaxDescent ();

	/*
	 * To center the text in the legend around the top of all
	 * characters and the baseline, also a spacing at the top of the
	 * characters with the descent length is minimal desired. So the
	 * font height should be not smaller as:
	 * 
	 *	 height  >=  ascent + (1 + frac_descent) * descent
	 *     
	 * with:
	 *
	 *	      frac_descent = TOP_DESCENT_PERC / 100
	 *
	 * Please note that no maximum ascent / descent values are used
	 * here.
	 */

    	int font_ascent  = font_metrics.getAscent  ();
	int font_descent = font_metrics.getDescent ();

	int total_descent = ( (100 + TOP_DESCENT_PERC) * font_descent )
			    / 100;

        if (font_height < font_ascent + total_descent)    // Height
        {						  // too
            font_height = font_ascent + total_descent;    // small.
        }

	/*
	 * The Y offset from the top of the characters in the font is:
	 *
	 *     offset = ( height - ascent -
	 *		  (1 + frac_descent) * descent ) / 2 +
	 *		frac_descent * descent + ascent 
	 *	      = ( height + ascent +
	 *		(frac_descent - 1) * descent ) / 2
	 */

	y_font_offset = ( 100 * (font_height + font_ascent) +
			  (TOP_DESCENT_PERC - 100) * font_descent )
			/ 200;

	/*
	 * Determine the maximum width from the legend labels.
	 */

	int  nr_labels	     = labels.size ();
	int  max_label_width = 0;

	for (int index = 0;  index < nr_labels;  index++)
	{
	    String  label	= (String) labels.elementAt (index);
	    int	    label_width = font_metrics.stringWidth  (label);

	    if (label_width > max_label_width)
		max_label_width = label_width;
	}

	/*
	 * Set the minimum width and the height of a row in the legend.
	 */

	if (legend_handler.getLegendUseMarkers ())  // Markers are used.
	{
	    /*
	     * Inquire the marker size from the plot canvas.
	     */

	    int  marker_size = PlotMarker.getSize ();

	    /*
	     * Set the minimum required width when markers are used. Add
	     * also border and marker-label separation.
	     */

	    min_size.width = marker_size + marker_label_separ +
			     max_label_width + 2 * border;

	    /*
	     * Set the row height to the maximum of font row height or
	     * marker height. Add also space for the border.
	     */

	    if (marker_size > font_height)
		row_height = marker_size + 2 * border;
	    else
		row_height = font_height + 2 * border;
	}
	else    // No markers are used.
	{
	    /*
	     * Mininimal width and row height follow straightforward
	     * from the maximum label width and font height with border
	     * space added.
	     */

	    min_size.width = max_label_width + 2 * border;
	    row_height	   = font_height     + 2 * border;
	}

	/*
	 * Set the minimum height as it follows from the row height and
	 * the row separation.
	 */

	min_size.height = nr_labels * (row_height + row_separ) -
			  row_separ;

    } // initSizes ()


    /**
     * Get the minimum size.
     *
     *    @return   The minimum size.
     *
    ***********************/
    Dimension  getMinSize ()
    /**********************/
    {
	return (min_size);
    }


    /**
     * Set the Y scroll value. It is used as offset for the Y
     * coordinates to deal with the scrollbar.
     *
     *    @param  y_scroll   The Y scroll value.
     *
    *******************************/
    void  setYScroll (int  y_scroll)
    /******************************/
    {
	this.y_scroll = y_scroll;
    }


    /**
     * Get the Y scroll value. It is used as offset for the Y
     * coordinates to deal with the scrollbar.
     *
     *    @return   The Y scroll value.
     *
    *****************/
    int  getYScroll ()
    /****************/
    {
	return y_scroll;
    }


    /**
     * When the mouse enters the legend canvas, call the
     * <CODE>legendMouseEnter()</CODE> method of the
     * <CODE>PlotLegendHandler</CODE> interface.
     *
     *    @param  event   The event hat caused the action.
     *
     *    @param  x	  The X-coordinate.
     *
     *    @param  y	  The Y-coordinate.
     *
     *    @return   True if the event has been handled and no further
     *		    action is necessary; false otherwise.
     *
    ****************************************/
    public boolean  mouseEnter (Event  event,
				int    x,
				int    y)
    /***************************************/
    {
	if (event.target == this)    // Target is the legend canvas.
	{
	    legend_handler.legendMouseEnter ();

	    return true;
	}
	else    // Another target: handled by the super method.
	{
	    return super.mouseEnter (event, x, y);
	}

    } // mouseEnter ()


    /**
     * When the mouse exits the legend canvas, call the
     * <CODE>legendMouseExit()</CODE> method of the
     * <CODE>PlotLegendHandler</CODE> interface.
     *
     *    @param  event   The event hat caused the action.
     *
     *    @param  x	  The X-coordinate.
     *
     *    @param  y	  The Y-coordinate.
     *
     *    @return   True if the event has been handled and no further
     *		    action is necessary; false otherwise.
     *
    ***************************************/
    public boolean  mouseExit (Event  event,
			       int    x,
			       int    y)
    /**************************************/
    {
	if (event.target == this)    // Target is the legend canvas.
	{
	    legend_handler.legendMouseExit ();

	    return true;
	}
	else    // Another target: handled by the super method.
	{
	    return super.mouseExit (event, x, y);
	}

    } // mouseExit ()


    /**
     * When the mouse is released in a legend box, call the
     * <CODE>legendMouseUp()</CODE> method of the
     * <CODE>PlotLegendHandler</CODE> interface to handle the selected
     * legend. The method is not called when the mouse was not in a
     * legedd box. Return if the mouse release has been handled.
     *
     *    @param  event   The event hat caused the action.
     *
     *    @param  x	  The X-coordinate.
     *
     *    @param  y	  The Y-coordinate.
     *
     *    @return   True if the event has been handled and no further
     *		    action is necessary; false otherwise.
     *
    *************************************/
    public boolean  mouseUp (Event  event,
			     int    x,
			     int    y)
    /************************************/
    {
	/*
	 * Let the parent handle the mouse release when the target of
	 * the event was not the legend canvas.
	 */

	if (event.target != this)
	    return super.mouseUp (event, x, y);

	/*
	 * Let the parent handle the mouse release when it was not
	 * within the legend canvas.
	 */
	 
	if ( x < 0  ||  x >= legend_size.width   ||
	     y < 0  ||  y >= legend_size.height )
	{
	    return super.mouseUp (event, x, y);
	}

	/*
	 * Let the parent handle the mouse release when the height of
	 * the mouse box was still undefined.
	 */

	if (row_height <= 0)
	    return super.mouseUp (event, x, y);

	/*
	 * Calculate the index of the legend box where the Y-coordinate
	 * resides. Set it to -1 when the mouse was not released within
	 * a legend box.
	 */

	int  index = yPixelToIndex (y);     // Match to box + separator.

	/*
	 * Check if the mouse was released in the separation area
	 * between two boxes.
	 */
	 
	int  y_box_bound = indexToYPixel (index) + row_height;

	if (y >= y_box_bound)    // In separation area.
	    index = -1;

	/*
	 * Call the mouse released method of the plot legend handler,
	 * specifying the legend where the mouse is released, if any.
	 * Call the "super()" method when the event was unused.
	 */

	if (legend_handler.legendMouseUp (event, index))    // Used.
	{
	    return true;
	}
	else    // Unused.
	{
	    return super.mouseUp (event, x, y);
	}

    } // mouseUp ()


    /**
     * Update the legend canvas. Because it will be repainted
     * completely, there is no need to clear in with
     * <CODE>clearRect</CODE>. This reduces flicker.
     *
     *    @param  g  The graphics context to use for updating.
     *
    ********************************/
    public void  update (Graphics  g)
    /*******************************/
    {
	paint (g);    // Repaint the complete legend.
    }


    /**
     * Paint the legend canvas.
     *
     *    @param  g   The graphics context to use for painting.
     *
    *******************************/
    public void  paint (Graphics  g)
    /******************************/
    {
	/*
	 * Get the legend sizes. Return when one of the sizes is zero.
	 */

	legend_size = size ();

	if (legend_size.width <= 0  || legend_size.height <= 0)
	    return;

	/*
	 * Get the overall background of the legends.
	 */

	Color  background  = getBackground (); 
	
	/*
	 * Inquire from the plot legend handler if markers should be
	 * used.
	 */
	 
	boolean  use_markers = legend_handler.getLegendUseMarkers ();

	/*
	 * Set the origin X coordinates for the legend rectangle, the
	 * coordinate in the legend window and the separation position.
	 */

	int  x_rect   = 0;
	int  x_legend = (legend_size.width -
			 min_size.width + 2 * border) / 2;
	int  x_separ  = 0;

	if (x_legend < border)    // Do not draw below the border.
	    x_legend = border;

	/*
	 * Set the origin X coordinates for the markers (when used) and
	 * for the labels.
	 */

	int  marker_size = PlotMarker.getSize ();
	int  x_marker	 = x_legend + marker_size / 2;
	int  x_label;

	if (use_markers)
	    x_label = x_legend + marker_size + marker_label_separ;
	else
	    x_label = x_legend;

	/*
	 * Set the Y coordinate offset from the border to the label.
	 * Adjust it when markers are used and when the marker height is
	 * larger than the row height.
	 */

	int  y_label_offset;

	if (use_markers  &&  marker_size > font_height)
	    y_label_offset = border + (marker_size - font_height) / 2;
	else
	    y_label_offset = border;

	/*
	 * Do nothing when one of the sizes in the graphics clip
	 * rectangle is zero.
	 */

	Rectangle  clip_rect = g.getClipRect ();

	if (clip_rect.width == 0  ||  clip_rect.height == 0)
	    return;

	/*
	 * Paint the legends which intersect with the clip rectangle.
	 * The intersection is determined in integral steps of
	 * row_height + row_separ.
	 */

	int  y_start	      = clip_rect.y;
	int  y_last	      = y_start + clip_rect.height - 1;
	int  start_index      = yPixelToIndex (y_start);
	int  last_index       = yPixelToIndex (y_last);
	int  last_label_index = labels.size   () - 1;

	/*
	 * Do for all plot label indices intersecting with the clip
	 * rectangle.
	 */

	for (int index = start_index;  index <= last_index;  index++)
	{
	    /*
	     * The origin Y coordinate is the coordinate of the plot
	     * label rectangle.
	     */

	    int	 y_rect = indexToYPixel (index);

	    /*
	     * Fill the plot label rectangle with the background for
	     * this legend. It is inquired from the plot legend handler.
	     */

	    g.setColor (legend_handler.getLegendBackground (index));
	    g.fillRect (x_rect, y_rect, legend_size.width, row_height);

	    /*
	     * Set the foreground color, inquired from the legend
	     * handler.
	     */

	    g.setColor (legend_handler.getLegendForeground (index));

	    /*
	     * Draw the plot trace marker if used. They are also
	     * obtained from the legend handler.
	     */

	    if (use_markers)
	    {
		int  y_marker = y_rect + row_height / 2;
		
		PlotMarker  marker =
		    legend_handler.getLegendPlotMarker (index);

		marker.draw (x_marker, y_marker, g);
	    }

	    /*
	     * Draw the plot trace label. Note that "y_label_offset"
	     * also counts for markers offset, if markers are used.
	     */

	    String  label   = (String) labels.elementAt (index);
	    int     y_label = y_rect + y_label_offset + y_font_offset;

	    g.drawString (label, x_label, y_label);

	    /*
	     * Draw the plot label separation rectangle with the default
	     * background. There is no separator after the last label in
	     * the legend.
	     */

	    if (index < last_label_index)
	    {
		int  y_separ = y_rect + row_height;

		g.setColor (background);
		g.fillRect (x_separ, y_separ,
			    legend_size.width, row_separ);
	    }
	
	} // For all plot label indices intersecting with the clip rect.

    } // paint ()


    /**
     * Convert an Y pixel coordinate to a plot label index. The
     * separator rectangle <EM>after</EM> the plot label is considered
     * to be part of the plot label.
     *
     *    @param  y  The Y coordinate.
     *
     *    @return   The index of the label rectangle.
     *
    **********************************/
    private int  yPixelToIndex (int  y)
    /*********************************/
    {
	int  nr_labels = labels.size ();  // The # labels in the legend.
	int  y_legend  = y + y_scroll;    // Correct for scrolling.

	/*
	 * The label index is the integral step of the label heigh +
	 * row separation height.
	 */

	int  index = y_legend / (row_height + row_separ);

	/*
	 * Return the index. Correct it when necessarily.
	 */

	if (index < 0)
	    return 0;
	else if (index >= nr_labels)
	    return nr_labels - 1;
	else
	    return index;
    
    } // yPixelToIndex ()


    /**
     * Convert a plot label index to an Y coordinate. It is the first
     * coordinate of the label rectangle. The separator rectangle
     * <EM>after</EM> the plot label is considered to be part of the
     * plot label.
     *
     *    @param  index   The index of the plot label.
     *
     *    @return   The first Y coordinate of the plot label.
     *
    **************************************/
    private int  indexToYPixel (int  index)
    /*************************************/
    {
	return index * (row_height + row_separ) - y_scroll;
    }

} // class LegendCanvas
