/* ------------------------------------------------------------------ */
/*		   N e t D a t a T a b l e . j a v a		      */
/*		   =================================		      */
/*								      */
/*   This class contains the source of the "NetDataTable" class.      */
/* ------------------------------------------------------------------ */


import java.util.*;


/**
 * This class is used to store the net data in a <CODE>Hashtable</CODE>
 * instance. It is also used to parse the fields to store from an input
 * line: the fields of this line are used to store the net data read in
 * the hashtable.
 * </P>
 *
 * <P>
 * The format of the hashtable is as follows: Data are stored from one
 * or more hosts which are charateristic for there host(s). From the
 * specified host(s) a key is generated. This is done with an instance
 * of the <CODE>GenHostKey</CODE> class. For this key a hashtable is
 * stored which contains all values where the keys are formed from the
 * dates and times.
 *
 *    @see  GenHostKey
 *
 *    @author  Hans Blom
 *
******************/
class  NetDataTable
/*****************/
{
    static final String  QOS  = "QoS";
    static final String  INET = "Inet";

    static final String  DEFAULT_NET_TYPE = INET;

    /*
     * Set the indices of the positions from the fields in an input
     * line.
     */

    private static final int  DATE_IND	     = 0;  // Date + time index.
    private static final int  VALUE_IND	     = 1;  // Net data val. ind.
    private static final int  START_HOST_IND = 2;  // First host index.

    /*
     * Set the class instance variables.
     */

    private Hashtable   net_data;	   // Hasht. for the net data.
    private Hashtable	no_plot_host_keys; // Hsht. h. keys not in plot.
    private GenHostKey	gen_host_key;	   // Used to get host keys.
    private String []   input_words;	   // Storage words in inp. ln.
    private String []	data_hosts;	   // Storage for data host ar.
    private DateFormat  input_date;	   // Storage for an input date.
    private int	        nr_hosts;	   // # hosts for these net dt.
    private boolean	sort_hosts;	   // Sort hosts to gener. key.


    /**
     * Constructor to initialise a class instance. If more than one host
     * is used, also the net type is defined in the data value;
     *
     *    @param  nr_hosts	 The #&nbsp;hosts which are defining a
     *				 set of date and time depedent net data
     *				 values.
     *
     *    @param  gen_host_key   This variable is used to generate host
     *				 keys from the specified host(s) and net
     *				 type.
     *
     *    @see  GenHostKey
     *
    **************************************/
    NetDataTable (int	      nr_hosts,
		  GenHostKey  gen_host_key)
    /*************************************/
    {
	this.nr_hosts	  = nr_hosts;
	this.gen_host_key = gen_host_key;

	int  nr_input_words = START_HOST_IND + nr_hosts;

	net_data	  = new Hashtable  ();
	no_plot_host_keys = new Hashtable  ();
	data_hosts	  = new String     [nr_hosts];
	input_words	  = new String     [nr_input_words];
	input_date	  = new DateFormat ();
    }


    /**
     * Return the hashtable containing the host keys of the net data
     * which will be not plotted.
     *
     *   @return   The hastable with the net host keys which are not
     *		   plotted.
     *
    ***********************************/
    Hashtable  getNoPlotHostKeysTable ()
    /**********************************/
    {
	return no_plot_host_keys;
    }


    /**
     * Remove all elements from the net data hashtable.
     *
    *************/
    void  clear ()
    /************/
    {
	net_data.clear ();
    }


    /**
     * Parse the specified input line and store the data read in the
     * net data hashtable.
     *
     *    @param  input_line     An input line from which the data have
     *			         to be retrieved.
     *
     *    @param  hosts	         Vector in which the hosts are stored.
     *
     *    @param  dates          Vector in which the dates and times are
     *			         stored and sorted.
     *
     *    @param  host_titles    The <CODE>HostTitleTable</CODE>
     *				 hashtable, where the host titles are
     *				 stored as a key of the host names. The
     *				 are only stored if the host title of
     *				 the corresponding host exists.
     *
     *	  @param  date_id_table  If not <CODE>null</CODE>, the time
     *				 field of the date should be interpreted
     *				 as a week or day ID.
     *
     *    @param  net_type       The used net type. When only one host
     *				 is used, the net type is overruled to
     *				 the default net type.
     *
     *    @exception  Exception   An exception is thrown after a bad
     *				  formatted input line.
     *
    ************************************************************/
    void  parseLine (CharInputLine   input_line,
		     StringVector    hosts,
		     DateVector	     dates,
		     HostTitleTable  host_titles,
		     Hashtable	     date_id_table,
		     String	     net_type)  throws  Exception
    /***********************************************************/
    {
	/*
	 * Split the input line into words. Spaces are used as
	 * separators.
	 */

	StringTools.split (input_words,
			   StringTools.SPACE_SEPARS,
			   input_line.getArray  (),
			   input_line.getOffset (),
			   input_line.getBound  ());

	/*
	 * Check if the host title is defined for all host(s) found in
	 * the input line. Do no add the data if one of the host titles
	 * is undefined. Add the hosts found to the array with data
	 * hosts for this input line. It is used for parsing the host
	 * key.
	 */

	boolean  all_hosts_found = true;
	int	 host_ind	 = 0;

	while (host_ind < nr_hosts  &&  all_hosts_found)
	{
	    String  host = input_words [host_ind + START_HOST_IND];

	    if (host_titles.get (host) == null)    // No title found.
	    {
		all_hosts_found = false;
	    }
	    else    // Add the host to the input line data host storage.
	    {
		data_hosts [host_ind++] = host;
	    }
	}

	/*
	 * Do not add the data when one or more hosts are unknown.
	 */

	if (!all_hosts_found)
	    return;

	/*
	 * Set the net type to the default type when only one host is
	 * specified.
	 */

	if (nr_hosts == 1)
	    net_type = DEFAULT_NET_TYPE;

	/*
	 * Convert the host names into a key value.
	 */

	String  host_key = gen_host_key.get (data_hosts, net_type);

	/*
	 * Define the date and time or week/day ID instance from the
	 * input word string. Return the date key.
	 */

	String  date_key = input_date.set (input_words [DATE_IND],
					   date_id_table);

	/*
	 * Unqiuely sort the date + time instance into the date vector.
	 * Create a new date instance when the previous one was added to
	 * the date vector.
	 */

	if (dates.sortInsertElement (input_date))
	    input_date = new DateFormat ();

	/*
	 * Put the net data value into the net data hashtable, using the
	 * host(s) key and date + time key.
	 */

	putValue (host_key, date_key, input_words [VALUE_IND]);

    } // parseLine ()


    /**
     * Put a net data value into the net data hashtable. The host(s) key
     * is used to select (or create if non-existing) a hashtable, where
     * the net data values for this (these) host(s) are stored, as a
     * function of the date key.
     *
     *    @param  host_key   The host key which represenst the hosts
     *			     which are responsible for this net data
     *			     value.
     *
     *    @param  date_key   The data key which stands for the date and
     *			     time when the value was obtained.
     *
     *    @param  value      The net data value to store.
     *
    ****************************************/
    private void  putValue (String  host_key,
			    String  date_key,
			    String  value)
    /***************************************/
    {
	/*
	 * Get the hashtable with the net data values as a function of
	 * the date and time.
	 */

	Hashtable  value_table = (Hashtable) net_data.get (host_key);

	/*
	 * Create a new value - date hashtable was found and store this
	 * table under the host(s) key.
	 */

	if (value_table == null)
	{
	    value_table = new Hashtable ();

	    net_data.put (host_key, value_table);
	}

	/*
	 * Add the value to the value - date hashtable.
	 */

	value_table.put (date_key, value);

    } // putValue ()


    /**
     * Get the net data value which was connected with the specified
     * host(s) and the specified date key.
     *
     *    @param  hosts	     The hosts which are responsible for the net
     *			     data value.
     *
     *    @param  date_key   The date key which represents the data when
     *			     the value was measured.
     *
     *    @return  The obtained net data value.
     *
    *************************************/
    String  getValue (String []  hosts,
		      String	 date_key,
		      String	 net_type)
    /************************************/
    {
	/*
	 * Convert the host key from the array with host(s).
	 */

	String  host_key = gen_host_key.get (hosts, net_type);

	/*
	 * Get the value - date hashtable associated with this (these)
	 * host(s).
	 */

	Hashtable  value_table = (Hashtable) net_data.get (host_key);

	if (value_table == null)   // No value - date hashtable  found.
	{
	    return null;
	}
	else   // Value - date hashtable found.
	{
	    /*
	     * Return the net data value at the date / time specified by
	     * the date key.
	     */

	    return (String) value_table.get (date_key);
	}
    
    } // getValue ()


    /**
     * Get the net data value which was connected with the specified
     * host(s) and the specified date.
     *
     *    @param  hosts   The hosts which are responsible for the net
     *			  data value.
     *
     *    @param  date    The date when the value was measured.
     *
     *    @return  The obtained net data value.
     *
    **************************************/
    String  getValue (String []   hosts,
		      DateFormat  date,
		      String	  net_type)
    /*************************************/
    {
	/*
	 * Return the value. The string value of the date is the host
	 * key.
	 */

	return getValue (hosts, date.toString (), net_type);
    }

} // class NetDataTable
