/* ------------------------------------------------------------------ */
/*		     P i n g F o r m a t . j a v a		      */
/*		     =============================		      */
/*								      */
/*   This file contains the source of the "PingFormat" class.	      */
/* ------------------------------------------------------------------ */


/**
 * This class defines the format of a ping value. It contains the
 * minimum, average and maximum ping values.
 *
 *    @author  Hans Blom
 *
***************/
class PingFormat
/**************/
{
    /*
     * Define the separator between the ping fields.
     */

    private static final char  PING_SEPAR = '/';

    private static final String  PLOT_PING_SEPAR = "&nbsp;" +
						   PING_SEPAR +
						   "&nbsp;";

    /*
     * Define the ping types and the # ping types.
     */

    static final int  MIN_TYPE  = 0;
    static final int  AVG_TYPE  = 1;
    static final int  MAX_TYPE  = 2;
    static final int  ALL_TYPE  = 3;
    static final int  LOST_TYPE = 4;

    private static final int  NR_PING_TYPES = 5;

    /*
     * Define the indices of the split ping fields and the # split ping
     * fields.
     */

    private static final int  MIN_IND	     = 0;
    private static final int  AVG_IND	     = 1;
    private static final int  MAX_IND	     = 2;
    private static final int  LOST_IND	     = 3;
    private static final int  NR_PING_FIELDS = 4;

    /*
     * Define the conversion array from type to field index.
     */

    private static final int []  TYPE_TO_FIELD_IND =
    {
	MIN_IND,    // <- MIN_TYPE
	AVG_IND,    // <- AVG_TYPE
	MAX_IND,    // <- MAX_TYPE
	-1,	    // <- ALL_TYPE     (Will be composed in "get()".)
	LOST_IND    // <- LOST_TYPE
    };

    /*
     * Define the array to store the split ping fields.
     */

    private String []  ping_fields;


    /**
     * Constructor. It allocates the ping fields array and initialises
     * the ping field values to zero.
     *
    ************/
    PingFormat ()
    /***********/
    {
	ping_fields = new String [NR_PING_FIELDS];

	for (int i = 0;  i < NR_PING_FIELDS;  i++)
	    ping_fields [i] = "0";
    }


    /**
     * Set the ping fields from an input string.
     *
     *    @param  ping_str   The string which contains the ping fields.
     *
     *    @exception  Exception   An execption is throws when the input
     *				  ping string has a bad format.
     *
    **********************************************/
    void  set (String  ping_str)  throws  Exception
    /*********************************************/
    {
	StringTools.split (ping_fields, PING_SEPAR, ping_str);
    }


    /**
     * Return the ping value with the specified type.
     *
     *	  @param  ping_type   The ping type.
     *
     *    @return  The ping type value.
     *
     *    @exception  IndexOutOfBoundsException   The specified ping
     *						  type is not in the
     *						  specified range.
     *
    **************************************************************/
    String  get (int  ping_type)  throws  IndexOutOfBoundsException
    /*************************************************************/
    {
	if (ping_type == ALL_TYPE)
	{
	    StringBuffer  ping_sb = new StringBuffer ();

	    ping_sb.append (ping_fields [MIN_IND]);
	    ping_sb.append (PLOT_PING_SEPAR);
	    ping_sb.append (ping_fields [AVG_IND]);
	    ping_sb.append (PLOT_PING_SEPAR);
	    ping_sb.append (ping_fields [MAX_IND]);

	    return ping_sb.toString ();
	}
	else if (ping_type < 0  ||  ping_type >= NR_PING_TYPES)
	{
	    throw new IndexOutOfBoundsException (
			  "Ping type out of bounds: " + ping_type);
	}

	return ping_fields [TYPE_TO_FIELD_IND [ping_type]];
    
    } // get ()


    /**
     * Return if a ping value is defined. This is the case when all ping
     * fields do not have &quot;<CODE>-</CODE>&quot; as first
     * character.
     *
     *    @return  The result of the test to control if the ping value
     *		   was defined.
     *
    ********************/
    boolean  isDefined ()
    /*******************/
    {
	boolean  defined = true;
	int	 i	 = 0;

	while (i < NR_PING_FIELDS  &&  defined)
	    if (ping_fields [i++].charAt (0) == '-')
		defined = false;

	return defined;
    }

} // class PingFormat
