# ----------------------------------------------------------------------
#	    host_net_test
#
#   Do the net performance test at this host and print the results to
#   Standard Output and Standard Error. This script is typically called
#   from a remote, controlling host which handles the net performance
#   measurements by running the script "net_test". The following values
#   are returned in the output streams:
#
#      o The load value of this host.
#
#      o The roundtrip values from this host to the hosts specified in
#	 the argument vector.
#
#      o The throughput values to those hosts. The program "netperf" is
#	 used to do these measurements.
#
#		Authors:  Mark Jansen,
#			  Hans Blom
# ----------------------------------------------------------------------


use strict;    # Restrict unsave constructs.

##
# Set Boolean constants.
##

my $FALSE = 0;
my $TRUE  = 1;

##
# Define the hashtables for the parameters used in this script.
##

my %var;    # The general script parameters;

##
# Initialise some variables to their default values.
##

init_variables (\%var);

##
# Read the program arguments and options.
##

read_arguments (\%var, \@ARGV);

##
# Print a usage message and exit when there are no hosts defined for the
# roundtrip and throughput tests.
##

if ($#{$var{'test_hosts'}} < $[)
{
    print_usage (\%var);
    exit 1;
}

##
# Do the throughput and roundtrip measurements. Print the results.
##

do_print_net_tests (\%var);

##
# This function initialises some variables with their default values. A
# larger part of these variables can also be defined using the script
# options.
#
####################
sub init_variables #
####################
#
# Function parameters:
{
    my $var_ref = shift;    # A reference to the general parameters.

    ##
    # Initialise some variables with their default values. A larger part
    # of the variables can be defined using the script options.
    ##

    $var_ref->{'ping_count'      } = 3;      # Total # ping values.
    $var_ref->{'skip_ping'       } = 1;      # Init. # ping v. to skip.
    $var_ref->{'timeout'	 } = 1000;   # Timeout wait ping [ms].
    $var_ref->{'ping_accept_perc'} = 80;     # Perc. # ping val. needed.
    $var_ref->{'ping_packet_size'} = 56;     # Ping packet size [bytes].
    $var_ref->{'use_qos'         } = $TRUE;  # Flag to do QoS net tests.
    $var_ref->{'test_flag'       } = $FALSE; # Flag for diagn. messages.
    $var_ref->{'inet_name'       } = 'Inet'; # Name usual Internet type.
    $var_ref->{'qos_name'        } = 'QoS';  # Name QoS net type.

} # sub init_variables

##
# This function parses the script options and arguments from the program
# argument vector.
#
####################
sub read_arguments #
####################
#
# Function parameters:
{
    my $var_ref	 = shift;    # A reference to the general parameters.
    my $argv_ref = shift;    # A reference to the argument vector.

    my $arg;

    while ($arg = shift (@$argv_ref))    # Get a new program argument.
    {
	if ($arg =~ /^-(?!\/)/)  # The arg. is an opt. (no opt.: -/...).
	{
	    if ($arg eq '-accept')    # Perc. of ping val. to acc. test.
	    {
		$var_ref->{'ping_accept_perc'} = shift (@$argv_ref);
	    }
	    elsif ($arg eq '-count')    # Set the # ping values.
	    {
		$var_ref->{'ping_count'} = shift (@$argv_ref);
	    }
	    elsif ($arg eq '-skip')  # Set the init. # skipped ping val.
	    {
		$var_ref->{'skip_ping'} = shift (@$argv_ref);
	    }
	    elsif ($arg eq '-packet_size')   # Set the ping packet size.
	    {
		$var_ref->{'ping_packet_size'} = shift (@$argv_ref);
	    }
	    elsif ($arg eq '-timeout') # Timeout to wait for each reply.
	    {
		$var_ref->{'timeout'} = shift (@$argv_ref);
	    }
	    elsif ($arg eq '-no_QoS') # No Quality-of-Service net avail.
	    {
		$var_ref->{'use_qos'}  = $FALSE;
	    }
	    elsif ($arg eq '-ip_qos')  # Set QoS gateway for host in ht.
	    {
		my $host;
		my $ip;

		( $host, $ip ) = split (/\//, shift (@$argv_ref));

		$var_ref->{'ip_qos_gws'}->{$host} = $ip;
	    }
	    elsif ($arg eq '-ip_inet') # Set Inet gateway for host in h.
	    {
		my $host;
		my $ip;
	    
		( $host, $ip ) = split (/\//, shift (@$argv_ref));

		$var_ref->{'ip_inet_gws'}->{$host} = $ip;
	    }
	    elsif ($arg eq '-stderr_file')  # Redirect "stderr" to file.
	    { 
		my $stderr_file = shift (@$argv_ref);

		open (STDERR, ">$stderr_file")  ||
		    die "$0: $!: Redirect stderr to: $stderr_file\n";
	    }
	    elsif ($arg eq '-test')   #  Give extra diagnostic messages.
	    {
		$var_ref->{'test_flag'} = $TRUE;
	    }
	    elsif ($arg eq '-help')    # Print a usage message and exit.
	    {
		print_usage ($var_ref);
		exit 1;
	    }
	    elsif ($arg eq '--'   ||  # Select "netperf" test spec. opt.
		   $arg =~ /^-D/  ||  # Set "netperf" "TCP_STREAM" opt.
		   $arg eq '-d')      # Set "netperf" debugging option.
	    {
		push (@{$var_ref->{'netperf_args'}}, $arg);
	    }
	    elsif ($arg eq '-h')    # Skip "netperf" usage option.
	    {
	    }
	    elsif ($arg eq '-H')    # Skip "netperf" remote host option.
	    {
		shift (@$argv_ref);
	    }
	    else    # Set all other "netperf" options with arguments.
	    {
		push (@{$var_ref->{'netperf_args'}}, $arg);
		push (@{$var_ref->{'netperf_args'}}, shift(@$argv_ref));
	    }
	}
	else  # Set a host to participate in the net performance tests.
	{
	    push (@{$var_ref->{'test_hosts'}}, $arg);
	}

    } # Parse the program options and arguments.

} # sub read_argument

##
# This function does the net performance tests to all specified hosts.
# The output is written to Standard Output.
#
########################
sub do_print_net_tests #
########################
#
# Function parameters:
{
    my $var_ref = shift;    # Reference to the general parameters.

    my $test_hosts_ref	= \@{$var_ref->{'test_hosts'}};
    my $ip_inet_gws_ref = \%{$var_ref->{'ip_inet_gws'}};
    my $ip_qos_gws_ref  = \%{$var_ref->{'ip_qos_gws'}};
    my $inet_name	= $var_ref->{'inet_name'};
    my $qos_name	= $var_ref->{'qos_name'};
    my $use_qos		= $var_ref->{'use_qos'};
    my $test_host;

    ##
    # Do the net performance measurements for all the specified hosts.
    ##

    foreach $test_host (@$test_hosts_ref)    # Get the next host info.
    {
	##
	# Split the host (dependent) information from the current host
	# string.
	##

	my $test_host_inet;
	my $test_host_qos;
	my $host_title;
	my $netperf_port;

	( $test_host_inet,
	  $test_host_qos,
	  $host_title,
	  $netperf_port ) = split (/\//, $test_host);

	##
	# Do the Internet tests only when the test host is not skipped.
	##

	if ($test_host_inet ne '-')    # Internet test host not skipped.
	{
	    ##
	    # Print the Internet hostname to be parsed by the reading
	    # script and/or for logging purposes.
	    ##
	    
	    print "\nTest_Host_", $inet_name, "  :=  ", $test_host_inet,
		  "\n";

	    ##
	    # Change the usual Internet hostname to a specified gateway,
	    # when defined.
	    ##
    
	    if (defined ($ip_inet_gws_ref->{$test_host_inet}))
	    {
		$test_host_inet = $ip_inet_gws_ref->{$test_host_inet};
	    }

	    ##
	    # Do the Internet throughput and roundtrip measurements.
	    ##

	    do_print_throughput ($var_ref,
				 $test_host_inet,
				 $netperf_port,
				 $inet_name);
	    do_print_roundtrip  ($var_ref,
				 $test_host_inet,
				 $inet_name);

	} # Internet test host not skipped.

	##
	# When specified and when the test is not skipped: do the
	# Quality-of-Service net tests in the same way as for the usual
	# Internet route.
	##

	if ($use_qos  &&  $test_host_qos ne '-')
	{
	    print "\nTest_Host_", $qos_name, "  :=  ", $test_host_qos,
		  "\n";

	    if (defined ($ip_qos_gws_ref->{$test_host_qos}))
	    {
		$test_host_qos = $ip_qos_gws_ref->{$test_host_qos};
	    }

	    do_print_throughput ($var_ref,
				 $test_host_qos,
				 $netperf_port,
				 $qos_name);
	    do_print_roundtrip	($var_ref,
				 $test_host_qos,
				 $qos_name); 

	} # Use QoS and test host not skipped.

    } # Do the net tests for all hosts.

} # sub do_print_net_tests

##
# Do the throughput measurements from this host to the specified test
# host. The throughput is measured with "netperf". The throughput values
# are printed to Standard Output.
#
#########################
sub do_print_throughput #
#########################
#
# Function parameters:
{
    my $var_ref      = shift;   # A reference to the general parameters.
    my $test_host    = shift;   # The target host for "netperf".
    my $netperf_port = shift;   # The port to use by "netperf".
    my $type_name    = shift;   # The name of the net type.

    my	  $netperf_args_ref = \@{$var_ref->{'netperf_args'}};
    my	  $test_flag	    = $var_ref->{'test_flag'};
    my	  $netperf_command;
    local *netperf_read;
    my	  @netperf_words;
    my	  $throughput;

    ##
    # Skip the test when the "netperf" port is less than zero: the
    # "netserver" server program could not be started at the target
    # host.
    ##

    if ($netperf_port <= 0)
    {
	if ($test_flag)
	{
	    print STDERR "No netserver for host: $test_host\n";
	}

	return;
    }

    ##
    # Construct the "netperf" command line. The option "-P 0" skips the
    # test banner.
    ##

    $netperf_command = 'netperf -H ' . $test_host . ' -p ' .
		       $netperf_port . " -P 0 @$netperf_args_ref";
 
    if ($test_flag)    # Diagnostics: print the "netperf" command line.
    {
        print "\n$netperf_command\n";
    }
    
    ##
    # Open the "netperf" command as an input filter. Exit at failure.
    ##
    
    open (netperf_read, "$netperf_command|") ||
	die "$0: $!: $netperf_command\n";

    ##
    # Read the "netperf" output and parse the output lines.
    ##
    
    while (<netperf_read>)
    {
	chop;    # Skip trailing newlines.

	##
	# Split the output line in space separated words: the last word
	# is the throughput value.
	##

	@netperf_words = split (/\s/);
	$throughput    = $netperf_words [$#netperf_words];
    }

    ##
    # Close the "netperf" filter.Give a diagnostic mesage at errors and
    # do not use the throughput value".
    ##

    close (netperf_read);

    if ($? != 0)
    {
	print STDERR
	      "$0: Error in netperf command: $!: $netperf_command: ",
	      "Ignored\n";
	return;
    }

    ##
    # Print the throughput value. Use a different value type name for a
    # corresponding network such that the caller can differentiate.
    ##
    
    print "Throughput_", $type_name, " :=  ", $throughput, "\n";

} # sub do_print_throughput

##
# Do the roundtrip measurements for the specified host and print the
# results. The minimum, average, maximum roundtrip times and the
# percentage package lost are printed.
#
########################
sub do_print_roundtrip #
########################
#
# Function parameters:
{
    my $var_ref   = shift;    # A reference to the general parameters.
    my $ping_host = shift;    # The target host for "ping".
    my $type_name = shift;    # The name of the net type.

    ##
    # Declare and set some help variables related with the ping
    # arguments.
    ##

    my $ping_accept_perc = $var_ref->{'ping_accept_perc'};
    my $ping_count       = $var_ref->{'ping_count'};
    my $ping_packet_size = $var_ref->{'ping_packet_size'};
    my $skip_ping        = $var_ref->{'skip_ping'};
    my $timeout		 = $var_ref->{'timeout'};

    ##
    # Declare and set other help variables.
    ##

    my $test_flag = $var_ref->{'test_flag'};

    ##
    # Declare and initialise some variables related with the "ping"
    # values.
    ##

    my $total_time  =  0.0;    # Sum of the roundtrip times.
    my $min_time    = -1.0;    # Minimum roundtrip time.
    my $avg_time    = -1.0;    # Average roundtrip time.
    my $max_time    = -1.0;    # Maximum roundtrip time.
    my $perc_lost   = -1.0;    # Percentage roundtrip values lost.
    my $time_number =  0;      # The # roundtrip val. counted until now.
    my $ping_number =  0;      # The non-skip. # roundtrip val. counted.
    my $time_ind    = $[ + 4;  # The ind. of the time v. in the time ar.

    my    $ping_nr_lim;
    my    $time;
    my    $ping_command;
    local *ping_read;
    my    @ping_words;
    my    @err_mess;

    ##
    # Calculate the minimum required, # non-skipped ping values from the
    # total # non-skipped ping values and the ping accept percentage.
    ##
    
    $ping_nr_lim = ($ping_accept_perc * ($ping_count - $skip_ping)) /
		   100;

    ##
    # Initialise the array containing the error messages from the
    # roundtrip output. Error messages are all ping lines which could
    # not be parsed correctly.
    ##

    @err_mess = ( );

    ##
    # Construct the ping command line.
    ##
    
    $ping_command = 'ping -l ' . $ping_packet_size . ' -n ' . $ping_count .
		     ' -w ' . $timeout . ' ' .  $ping_host;

    ##
    # Diagnostics: print the roundtrip command to execute.
    ##

    if ($test_flag)
    {
        print "\n$ping_command\n";
    }

    ##
    # Open the roundtrip command as a filter command to read the output
    # from.
    ##
    
    open (ping_read, "$ping_command|")  ||
	die "$0: $!: $ping_command\n";

    ##
    # Read the roundtrip output and parse the output lines.
    ##

    while (<ping_read>)
    {
	##
	# Split the output line in space separated words.
	##

	@ping_words = split (/\s/);

	##
	# Replace the time prefix and the time unit in the time string:
	# The remaining time value is placed in the first replaced
	# positional parameter.
	##

	if ( $#ping_words >= $time_ind  &&
	     $ping_words[$time_ind] =~ /^time(.*)ms$/ )
	{
	    ##
	    # Store the time from the first substituted positional
	    # parameter.
	    ##

	    $time = $1;

	    ##
	    # Only accept the time when a roundtrip value could be
	    # obtained. Set upper bounded times to zero.
	    ##

	    if ($time =~ /^</)   # Upper boundary roundtrip time.
	    {
		$time = 0;
	    }
	    else    # Roundtrip time could be obtained.
	    {
		##
		# Skip the leading '=' character from the valid
		# roundtrip time value.
		##

		$time =~ s/^=//;
	    }

	    ##
	    # Only accept values after the initial # pings skipped.
	    ##

	    if ($time_number < $skip_ping)    # Skip ping value.
	    {
		if ($test_flag)    # Diagnostics: print skipped value.
		{
		    print "Ping      ", $time_number, ": T=", $time,
			  "  (Skipped)\n";
		}
	    }
	    else    # Ping value accepted.
	    {
		##
		# Determine the ping extrema.
		##

		if ($min_time < 0.0)    # Init. extrema at 1th value.
		{
		    $min_time = $time;
		    $max_time = $time;
		}
		elsif ($time < $min_time)    # New minimum.
		{
		    $min_time = $time;
		}
		elsif ($time > $max_time)    # New maximum.
		{
		    $max_time = $time;
		}
	
		if ($test_flag)    # Diagnostics: print count and value.
		{ 
		    print "Ping      ", $time_number, ": T=", $time,
			  "\n";
		}

		##
		# Add the roundtrip time to the sum and increment the
		# non-skipped # roundtrip values for the calculation of
		# the average.
		##

		$total_time += $time;
		$ping_number++;

	    } # Ping value accepted.

	    $time_number++;    # Count the all roundtrip values.
	}
	else    # Incorrect time string: add line to error mesages.
	{
	    push (@err_mess, $_);
	}

    } # Read all roundtrip lines.

    $time_number = 0;

    ##
    # Close the roundtrip input stream. Give a diagnostic mesage at non-
    # zero exit values from the filter command. Use in this situation
    # only the roundtrip when the non-skipped # values is equal or
    # larger than the required minimum values. Note that roundtrip
    # values lost always leads to a non-zero exit value!
    ##

    close (ping_read);

    if ($? ne 0)    # Ping values lost / error.
    {
	if ($ping_number >= $ping_nr_lim)    # Accept the ping values.
	{
	    ##
	    # Give a diagnostic message of the values lost and that the
	    # values are accepted anyhow.
	    ##

	    print STDERR
		  "$0: Error in ping command: $ping_command: $!\n",
		  "Ping counts (", $ping_number, ") above limit: ping ",
		  "accepted\n",
		  "\n",
		  "Diagnostic messages:\n",
		  "\n",
		  "@err_mess";
	}
	else    # Do not accept the roundtrip values.
	{
	    ##
	    # Give a diagnostic message that the ping values are not
	    # accepted and skip them.
	    ##

	    print STDERR
	      	  "$0: Error in ping command: $ping_command: $!: ",
		  "Ignored\n",
		  "Ping counts (", $ping_number, ") below limit\n",
		  "Diagnostic messages:\n",
		  "\n",
		  "@err_mess";

	    return;
	}
    }
    elsif ($ping_number < $ping_nr_lim)    # No errors, but below limit.
    {
	print STDERR
	      "$0: # Ping counts (", $ping_number, ") below limit\n";

	return;

    } # No errors, but the # roundtrip counts is below the limit.

    if ($test_flag)
    {
	print "\n";
    }

    ##
    # Calculate the available roundtrip time when there are values
    # available.
    ##

    if ($ping_number > 0)
    {
	$avg_time = (1.0 * $total_time) / $ping_number;
    }
    else
    {
	$avg_time = 0.0;
    }

    ##
    # Calculate the percentage roundtrip values lost from the
    # non-skipped ping values.
    ##

    $perc_lost = ( 100.0 * ($ping_count - $ping_number - $skip_ping) ) /
		 $ping_count;

    ##
    # Print the various ping type values with a different header for
    # the corresponding net type.
    ##
    
    printf "Ping_Time_%s  :=  %.3f/%.3f/%.3f/%.3f\n",
	   $type_name, $min_time, $avg_time, $max_time, $perc_lost;

} # sub do_print_roundtrip

##
# Prints the different options that can be used.
#
#################
sub print_usage #
#################
#
# Function parameters:
{
    my $var_ref = shift;    # A reference to the general parameters.
    
    print STDERR
	  "Usage: $0 [<option> [...]] <ping_host> [...]\n",
	  "\n",
	  "Possible options:\n",
	  "\n",
	  "    -accept <ping_accept_percentage>\n",
	  "          Set the count percentage of pings to be accepted ",
	  "after\n",
	  "          package lost. Default: ",
	  $var_ref->{'ping_accept_perc'}, "\n",
	  "\n",
	  "    -count <ping_count>\n",
	  "          Set the # pings. Default: ",
	  $var_ref->{'ping_count'}, "\n",
	  "\n",
	  "    -skip <skip_ping>\n",
	  "          Set the initial # pings to skip. Default: ",
	  $var_ref->{'skip_ping'}, "\n",
	  "\n",
	  "    -packet_size <ping_packet_size [bytes]>\n",
	  "          Set the ping packet size. Default: ",
	  $var_ref->{'ping_packet_size'}, " bytes\n",
	  "\n",
	  "    -timeout <timeout>\n",
	  "          Specify the timeout in milliseconds to wait for ",
	  "each\n",
	  "          reply. Default: ", $var_ref->{'timeout'},
	  " [ms]\n",
	  "\n",
	  "    -no_QoS\n",
	  "          Do no Quality of Service tests.\n",
	  "\n",
	  "    -stderr_file <file>\n",
	  "          Redirect Standard Error to the specified file.\n",
	  "\n",
	  "    -test\n",
	  "          Print some extra information to Standard Error.\n",
	  "\n",
	  "Format <ping_host>:\n",
	  "\n",
	  "    <IP_host>/<QoS_host>/<host_title>\n";

} # sub print_usage
